# ***************************************************************************
# *   Copyright (c) 2022 Wanderer Fan <wandererfan@gmail.com>               *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides the TechDraw ShareView GuiCommand."""

__title__ = "TechDrawTools.CommandShareView"
__author__ = "WandererFan"
__url__ = "https://www.freecad.org"
__version__ = "00.01"
__date__ = "2022/01/11"

from PySide.QtCore import QT_TRANSLATE_NOOP

import FreeCAD as App
import FreeCADGui as Gui

import TechDrawTools

class CommandShareView:
    """Shares a view on current page to another page."""

    def __init__(self):
        """Initialize variables for the command that must exist at all times."""
        pass

    def GetResources(self):
        """Return a dictionary with data that will be used by the button or menu item."""
        return {'Pixmap': 'actions/TechDraw_ShareView.svg',
                'Accel': "",
                'MenuText': QT_TRANSLATE_NOOP("TechDraw_ShareView", "Share View"),
                'ToolTip': QT_TRANSLATE_NOOP("TechDraw_ShareView", "Shares a view on a second page")}

    def Activated(self):
        """Run the following code when the command is activated (button press)."""
#        print("Activated()")
        sel = Gui.Selection.getSelection()

        vName = ""
        views = list()
        for o in sel:
            if o.isDerivedFrom("TechDraw::DrawView"):
                views.append(o)
        if views:
            vName = views[0].Name

        toPageName = ""
        fromPageName = ""
        pages = list()
        for o in sel:
            if o.isDerivedFrom("TechDraw::DrawPage"):
                pages.append(o)
        if pages:
            fromPageName = pages[0].Name
        if len(pages) > 1:
            toPageName = pages[1].Name

        self.ui  = TechDrawTools.TaskShareView()

        self.ui.setValues(vName, fromPageName, toPageName)
        Gui.Control.showDialog(self.ui)

    def IsActive(self):
        """Return True when the command should be active or False when it should be disabled (greyed)."""
        if App.ActiveDocument:
            return TechDrawTools.TDToolsUtil.havePage() and TechDrawTools.TDToolsUtil.haveView()
        else:
            return False


#
# The command must be "registered" with a unique name by calling its class.
Gui.addCommand('TechDraw_ShareView', CommandShareView())




# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2024 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************

"""BIM equipment commands"""

import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP
translate = FreeCAD.Qt.translate

PARAMS = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/BIM")


class Arch_Equipment:
    "the Arch Equipment command definition"

    def GetResources(self):

        return {
            "Pixmap": "Arch_Equipment",
            "MenuText": QT_TRANSLATE_NOOP("Arch_Equipment", "Equipment"),
            "Accel": "E, Q",
            "ToolTip": QT_TRANSLATE_NOOP(
                "Arch_Equipment", "Creates an equipment from a selected object (Part or Mesh)"
            ),
        }

    def IsActive(self):

        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):

        s = FreeCADGui.Selection.getSelection()
        if not s:
            FreeCAD.Console.PrintError(
                translate("Arch", "Select a base shape object and optionally a mesh object")
            )
        else:
            base = ""
            mesh = ""
            if len(s) == 2:
                if hasattr(s[0], "Shape"):
                    base = s[0].Name
                elif s[0].isDerivedFrom("Mesh::Feature"):
                    mesh = s[0].Name
                if hasattr(s[1], "Shape"):
                    if mesh:
                        base = s[1].Name
                elif s[1].isDerivedFrom("Mesh::Feature"):
                    if base:
                        mesh = s[1].Name
            else:
                if hasattr(s[0], "Shape"):
                    base = s[0].Name
                elif s[0].isDerivedFrom("Mesh::Feature"):
                    mesh = s[0].Name
            FreeCAD.ActiveDocument.openTransaction(str(translate("Arch", "Create Equipment")))
            FreeCADGui.addModule("Arch")
            if base:
                base = "FreeCAD.ActiveDocument." + base
            FreeCADGui.doCommand("obj = Arch.makeEquipment(" + base + ")")
            if mesh:
                FreeCADGui.doCommand("obj.HiRes = FreeCAD.ActiveDocument." + mesh)
            FreeCADGui.addModule("Draft")
            FreeCADGui.doCommand("Draft.autogroup(obj)")
            FreeCAD.ActiveDocument.commitTransaction()
            FreeCAD.ActiveDocument.recompute()
            # get diffuse color info from base object
            if base and hasattr(s[0].ViewObject, "DiffuseColor"):
                FreeCADGui.doCommand(
                    "FreeCAD.ActiveDocument.Objects[-1].ViewObject.DiffuseColor = "
                    + base
                    + ".ViewObject.DiffuseColor"
                )
        return


FreeCADGui.addCommand("Arch_Equipment", Arch_Equipment())



# SPDX-License-Identifier: LGPL-2.1-or-later
# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2025 Billy Huddleston <billy@ivdc.com>                  *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************


import FreeCAD
import Path
from typing import Dict, Any, Optional, Union


class ParameterAccessor:
    """
    Unified accessor for dicts and FreeCAD objects for migration logic.
    """

    def __init__(self, target):
        self.target = target
        self.is_dict = isinstance(target, dict)

    def has(self, key):
        if self.is_dict:
            # For dicts, check in nested 'parameter' dict
            param = self.target.get("parameter", {})
            return key in param if isinstance(param, dict) else False
        else:
            return key in getattr(self.target, "PropertiesList", [])

    def get(self, key):
        if self.is_dict:
            # For dicts, get from nested 'parameter' dict
            param = self.target.get("parameter", {})
            return param.get(key) if isinstance(param, dict) else None
        else:
            return self.target.getPropertyByName(key)

    def set(self, key, value):
        if self.is_dict:
            # For dicts, set in nested 'parameter' dict
            if "parameter" not in self.target:
                self.target["parameter"] = {}
            self.target["parameter"][key] = value
        else:
            setattr(self.target, key, value)

    def add_property(self, prop_type, key, group, doc):
        if self.is_dict:
            # For dicts, just set the value
            pass  # No-op, handled by set()
        else:
            self.target.addProperty(prop_type, key, group, doc)

    def set_editor_mode(self, key, mode):
        if self.is_dict:
            pass  # No-op
        else:
            self.target.setEditorMode(key, mode)

    def label(self):
        if self.is_dict:
            return "toolbit"
        else:
            return getattr(self.target, "Label", "unknown toolbit")

    def get_shape_type(self):
        if self.is_dict:
            # For dicts, shape-type is at top level of attrs dict
            return self.target.get("shape-type")
        else:
            # For FreeCAD objects, use ShapeType attribute
            return getattr(self.target, "ShapeType", None)


if False:
    Path.Log.setLevel(Path.Log.Level.DEBUG, Path.Log.thisModule())
    Path.Log.trackModule(Path.Log.thisModule())
else:
    Path.Log.setLevel(Path.Log.Level.INFO, Path.Log.thisModule())


def migrate_parameters(accessor: ParameterAccessor) -> bool:
    """
    Migrates legacy parameters using a unified accessor.

    Currently handles:
    - TorusRadius → CornerRadius
    - FlatRadius/Diameter → CornerRadius

    Args:
        accessor: ParameterAccessor instance wrapping dict or FreeCAD object

    Returns:
        True if migration occurred, False otherwise
    """
    has_torus = accessor.has("TorusRadius")
    has_flat = accessor.has("FlatRadius")
    has_diam = accessor.has("Diameter")
    has_corner = accessor.has("CornerRadius")
    label = accessor.label()
    shape_type = accessor.get_shape_type()

    # Only run migration logic if shape type == 'Bullnose'
    if shape_type and str(shape_type).lower() == "bullnose":
        # Case 1: TorusRadius exists, copy to CornerRadius
        if has_torus and not has_corner:
            value = accessor.get("TorusRadius")
            accessor.add_property(
                "App::PropertyLength",
                "CornerRadius",
                "Shape",
                "Corner radius copied from TorusRadius",
            )
            accessor.set_editor_mode("CornerRadius", 0)
            accessor.set("CornerRadius", value)
            Path.Log.info(f"Copied TorusRadius to CornerRadius={value} for {label}")
            return True

        # Case 2: FlatRadius and Diameter exist, calculate CornerRadius
        if has_flat and has_diam and not has_corner:
            try:
                diam_raw = accessor.get("Diameter")
                flat_raw = accessor.get("FlatRadius")
                diameter = FreeCAD.Units.Quantity(diam_raw)
                flat_radius = FreeCAD.Units.Quantity(flat_raw)
                corner_radius = (float(diameter) / 2.0) - float(flat_radius)

                # Convert to correct unit
                if isinstance(diam_raw, str) and diam_raw.strip().endswith("in"):
                    cr_in = FreeCAD.Units.Quantity(f"{corner_radius} mm").getValueAs("in")
                    value = f"{float(cr_in):.4f} in"
                else:
                    if isinstance(diam_raw, str) and not diam_raw.strip().endswith("mm"):
                        cr_mm = FreeCAD.Units.Quantity(f"{corner_radius} in").getValueAs("mm")
                        value = f"{float(cr_mm):.4f} mm"
                    else:
                        value = f"{float(corner_radius):.4f} mm"

                accessor.add_property(
                    "App::PropertyLength",
                    "CornerRadius",
                    "Shape",
                    "Corner radius migrated from FlatRadius/Diameter",
                )
                accessor.set_editor_mode("CornerRadius", 0)
                accessor.set("CornerRadius", value)
                Path.Log.info(f"Migrated FlatRadius/Diameter to CornerRadius={value} for {label}")
                return True
            except Exception as e:
                Path.Log.error(f"Failed to migrate FlatRadius for toolbit {label}: {e}")
                return False

    return False



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2021 sliptonic <shopinthewoods@gmail.com>               *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

import FreeCAD as App
import Path
import Path.Base.Drillable as Drillable
import CAMTests.PathTestUtils as PathTestUtils


if False:
    Path.Log.setLevel(Path.Log.Level.DEBUG, Path.Log.thisModule())
    Path.Log.trackModule(Path.Log.thisModule())
else:
    Path.Log.setLevel(Path.Log.Level.INFO, Path.Log.thisModule())


class TestPathDrillable(PathTestUtils.PathTestBase):
    def setUp(self):
        App.ConfigSet("SuppressRecomputeRequiredDialog", "True")
        self.doc = App.open(App.getHomePath() + "/Mod/CAM/CAMTests/Drilling_1.FCStd")
        App.ConfigSet("SuppressRecomputeRequiredDialog", "")
        self.obj = self.doc.getObject("Pocket011")

    def tearDown(self):
        App.closeDocument(self.doc.Name)

    def test00(self):
        """Test CompareVecs"""

        # Vec and origin
        v1 = App.Vector(0, 0, 10)
        v2 = App.Vector(0, 0, 0)
        self.assertTrue(Drillable.compareVecs(v1, v2))

        # two valid vectors
        v1 = App.Vector(0, 10, 0)
        v2 = App.Vector(0, 20, 0)
        self.assertTrue(Drillable.compareVecs(v1, v2))

        # two valid vectors not aligned
        v1 = App.Vector(0, 10, 0)
        v2 = App.Vector(10, 0, 0)
        self.assertFalse(Drillable.compareVecs(v1, v2))

    def test10(self):
        """Test isDrillable"""

        # Invalid types
        candidate = self.obj.getSubObject("Vertex1")
        self.assertRaises(TypeError, lambda: Drillable.isDrillable(self.obj.Shape, candidate))

        # Test cylinder faces

        # thru-hole
        candidate = self.obj.getSubObject("Face30")

        # Typical drilling
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate))

        # Drilling with smaller bit
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, tooldiameter=20))

        # Drilling with bit too large
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate, tooldiameter=30))

        # off-axis hole
        candidate = self.obj.getSubObject("Face44")

        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # Passing explicit vector
        self.assertTrue(
            Drillable.isDrillable(self.obj.Shape, candidate, vector=App.Vector(0, -1, 0))
        )

        # Drilling with smaller bit
        self.assertTrue(
            Drillable.isDrillable(
                self.obj.Shape, candidate, tooldiameter=10, vector=App.Vector(0, -1, 0)
            )
        )

        # Drilling with bit too large
        self.assertFalse(
            Drillable.isDrillable(
                self.obj.Shape, candidate, tooldiameter=30, vector=App.Vector(0, -1, 0)
            )
        )

        # ellipse hole
        candidate = self.obj.getSubObject("Face29")

        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # raised cylinder
        candidate = self.obj.getSubObject("Face32")

        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # cylinder on slope
        candidate = self.obj.getSubObject("Face24")
        # Typical drilling
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # Circular Faces
        candidate = self.obj.getSubObject("Face54")

        # Typical drilling
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # Passing explicit vector
        self.assertTrue(
            Drillable.isDrillable(self.obj.Shape, candidate, vector=App.Vector(0, 0, 1))
        )

        # Drilling with smaller bit
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, tooldiameter=10))

        # Drilling with bit too large
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate, tooldiameter=30))

        # off-axis circular face hole
        candidate = self.obj.getSubObject("Face58")

        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # Passing explicit vector
        self.assertTrue(
            Drillable.isDrillable(self.obj.Shape, candidate, vector=App.Vector(0, -1, 0))
        )

        # raised face
        candidate = self.obj.getSubObject("Face49")
        # Typical drilling
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # interrupted Face
        candidate = self.obj.getSubObject("Face50")
        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # donut face
        candidate = self.obj.getSubObject("Face48")
        # Typical drilling
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # Test edges
        # circular edge
        candidate = self.obj.getSubObject("Edge55")

        # Typical drilling
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # Passing explicit vector
        self.assertTrue(
            Drillable.isDrillable(self.obj.Shape, candidate, vector=App.Vector(0, 0, 1))
        )

        # Drilling with smaller bit
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, tooldiameter=10))

        # Drilling with bit too large
        self.assertFalse(
            Drillable.isDrillable(self.obj.Shape, candidate, tooldiameter=30, vector=None)
        )

        # off-axis circular edge
        candidate = self.obj.getSubObject("Edge74")

        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertTrue(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # Passing explicit vector
        self.assertTrue(
            Drillable.isDrillable(self.obj.Shape, candidate, vector=App.Vector(0, 1, 0))
        )

        # incomplete circular edge
        candidate = self.obj.getSubObject("Edge39")
        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

        # elliptical edge
        candidate = self.obj.getSubObject("Edge56")
        # Typical drilling
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate))

        # Passing None as vector
        self.assertFalse(Drillable.isDrillable(self.obj.Shape, candidate, vector=None))

    def test20(self):
        """Test getDrillableTargets"""
        results = Drillable.getDrillableTargets(self.obj)
        self.assertEqual(len(results), 15)

        results = Drillable.getDrillableTargets(self.obj, vector=None)
        self.assertEqual(len(results), 20)

        results = Drillable.getDrillableTargets(self.obj, ToolDiameter=20, vector=None)
        self.assertEqual(len(results), 5)



# ***************************************************************************
# *   Copyright (c) 2011 Juergen Riegel <FreeCAD@juergen-riegel.net>        *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

import unittest

import FreeCAD


class TestThickness(unittest.TestCase):
    def setUp(self):
        self.Doc = FreeCAD.newDocument("PartDesignTestThickness")

    def testReversedThickness(self):
        self.Body = self.Doc.addObject("PartDesign::Body", "Body")
        self.Box = self.Doc.addObject("PartDesign::AdditiveBox", "Box")
        self.Body.addObject(self.Box)
        self.Box.Length = 10.00
        self.Box.Width = 10.00
        self.Box.Height = 10.00
        self.Doc.recompute()
        self.Thickness = self.Doc.addObject("PartDesign::Thickness", "Thickness")
        self.Thickness.Base = (self.Box, ["Face1"])
        self.Body.addObject(self.Thickness)
        self.Doc.recompute()
        self.Thickness.Value = 1.0
        self.Thickness.Reversed = 1
        self.Thickness.Mode = 0
        self.Thickness.Join = 0
        self.Thickness.Base = (self.Box, ["Face1"])
        self.Doc.recompute()
        self.assertEqual(len(self.Thickness.Shape.Faces), 11)
        # 6 faces of outer box + 4 faces of inner box + 16 edges outer, 8 inner,
        # + 8 vertexes outer, 8 inner + 1 solid = 51
        self.assertEqual(self.Thickness.Shape.ElementMapSize, 51)

    def tearDown(self):
        # closing doc
        FreeCAD.closeDocument("PartDesignTestThickness")
        # print ("omit closing document for debugging")



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2025 Jakub Michalski <jakub.j.michalski[at]gmail.com>         *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************
# to run the example use:
"""
from femexamples.equation_staticcurrent_elmer import setup
setup()

"""


import sys
import FreeCAD

import Fem
import ObjectsFem

from BasicShapes import Shapes
from . import manager
from .manager import get_meshname
from .manager import init_doc
from .meshes import generate_mesh


def get_information():
    return {
        "name": "Joule heating",
        "meshtype": "solid",
        "meshelement": "Tet10",
        "constraints": ["electrostatic potential", "current density"],
        "solvers": ["elmer"],
        "material": "solid",
        "equations": ["static current"],
    }


def get_explanation(header=""):
    return (
        header
        + """

To run the example from Python console use:
from femexamples.equation_staticcurrent_elmer import setup
setup()

Static current equation - Elmer solver

analytical solution - temperature 351.09 K at 50 s

"""
    )


def setup(doc=None, solvertype="elmer"):

    # init FreeCAD document
    if doc is None:
        doc = init_doc()

    # explanation object
    # just keep the following line and change text string in get_explanation method
    manager.add_explanation_obj(doc, get_explanation(manager.get_header(get_information())))

    # geometric objects

    # wire
    Wire = doc.addObject("Part::Cylinder", "Wire")
    Wire.Radius = "2 mm"
    Wire.Height = "60 mm"
    Wire.ViewObject.Visibility = True

    if FreeCAD.GuiUp:
        Wire.ViewObject.Document.activeView().viewAxonometric()
        Wire.ViewObject.Document.activeView().fitAll()

    # analysis
    analysis = ObjectsFem.makeAnalysis(doc, "Analysis")
    if FreeCAD.GuiUp:
        import FemGui

        FemGui.setActiveAnalysis(analysis)

    # solver
    if solvertype == "elmer":
        solver_obj = ObjectsFem.makeSolverElmer(doc, "SolverElmer")
        solver_obj.SimulationType = "Transient"
        solver_obj.TimestepSizes = [50]
        solver_obj.TimestepIntervals = [1]
        eq_staticcurrent = ObjectsFem.makeEquationStaticCurrent(doc, solver_obj)
        eq_staticcurrent.References = [(Wire, "Solid1")]
        eq_staticcurrent.CalculateJouleHeating = True
        eq_staticcurrent.HeatSource = True
        eq_staticcurrent.Priority = 2
        eq_heat = ObjectsFem.makeEquationHeat(doc, solver_obj)
        eq_heat.References = [(Wire, "Solid1")]
        eq_heat.Priority = 1
    else:
        FreeCAD.Console.PrintWarning(
            "Unknown or unsupported solver type: {}. "
            "No solver object was created.\n".format(solvertype)
        )
    analysis.addObject(solver_obj)

    # material

    # metal wire
    material_obj = ObjectsFem.makeMaterialSolid(doc, "Metal for Joule heating")
    mat = material_obj.Material
    mat["Name"] = "Metal for Joule heating"
    mat["Density"] = "7850 kg/m^3"
    mat["ThermalConductivity"] = "16.0 W/m/K"
    mat["SpecificHeat"] = "490.0 J/kg/K"
    mat["ElectricalConductivity"] = "1450000 S/m"
    material_obj.Material = mat
    material_obj.References = [(Wire, "Solid1")]
    analysis.addObject(material_obj)

    # voltage on one end
    Voltage = ObjectsFem.makeConstraintElectrostaticPotential(doc, "Voltage")
    Voltage.References = [(Wire, "Face2")]
    Voltage.Potential = "0 V"
    analysis.addObject(Voltage)

    # current on other end
    Current = ObjectsFem.makeConstraintCurrentDensity(doc, "Current")
    Current.References = [(Wire, "Face3")]
    Current.Mode = "Normal"
    Current.NormalCurrentDensity_re = "2387324 A/m^2"
    analysis.addObject(Current)

    # constraint initial temperature
    con_inittemp = ObjectsFem.makeConstraintInitialTemperature(doc, "ConstraintInitialTemperature")
    con_inittemp.initialTemperature = 300.0
    analysis.addObject(con_inittemp)

    # mesh
    femmesh_obj = analysis.addObject(ObjectsFem.makeMeshGmsh(doc, get_meshname()))[0]
    femmesh_obj.Shape = Wire
    femmesh_obj.ElementOrder = "2nd"
    femmesh_obj.CharacteristicLengthMax = "0.7 mm"
    femmesh_obj.ViewObject.Visibility = False

    # generate the mesh
    generate_mesh.mesh_from_mesher(femmesh_obj, "gmsh")

    doc.recompute()
    return doc



# SPDX-License-Identifier: LGPL-2.1-or-later

#***************************************************************************
#*   Copyright (c) 2002,2003 Juergen Riegel <juergen.riegel@web.de>        *
#*                                                                         *
#*   This file is part of the FreeCAD CAx development system.              *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU Lesser General Public License (LGPL)    *
#*   as published by the Free Software Foundation; either version 2 of     *
#*   the License, or (at your option) any later version.                   *
#*   for detail see the LICENCE text file.                                 *
#*                                                                         *
#*   FreeCAD is distributed in the hope that it will be useful,            *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU Lesser General Public License for more details.                   *
#*                                                                         *
#*   You should have received a copy of the GNU Library General Public     *
#*   License along with FreeCAD; if not, write to the Free Software        *
#*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#*   USA                                                                   *
#*                                                                         *
#***************************************************************************/

# Idf init module
#
# Gathering all the information to start FreeCAD
# This is the second one of three init scripts, the third one
# runs when the gui is up

# two options for IDF added by Milos Koutny (12-Feb-2010)
FreeCAD.addImportType("IDF emn file File Type (*.emn *.EMN)","Idf")
#FreeCAD.addImportType("IDF emp File Type (*.emp)","Import_Emp")



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2024 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************

"""BIM Schedule command"""

import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP
translate = FreeCAD.Qt.translate

PARAMS = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/BIM")


class Arch_SectionPlane:
    "the Arch SectionPlane command definition"

    def GetResources(self):

        return {
            "Pixmap": "Arch_SectionPlane",
            "Accel": "S, E",
            "MenuText": QT_TRANSLATE_NOOP("Arch_SectionPlane", "Section Plane"),
            "ToolTip": QT_TRANSLATE_NOOP(
                "Arch_SectionPlane",
                "Creates a section plane object, including the selected objects",
            ),
        }

    def IsActive(self):

        v = hasattr(FreeCADGui.getMainWindow().getActiveWindow(), "getSceneGraph")
        return v

    def Activated(self):

        sel = FreeCADGui.Selection.getSelection()
        ss = "["
        for o in sel:
            if len(ss) > 1:
                ss += ","
            ss += "FreeCAD.ActiveDocument." + o.Name
        ss += "]"
        FreeCAD.ActiveDocument.openTransaction(translate("Arch", "Create Section Plane"))
        FreeCADGui.addModule("Arch")
        FreeCADGui.doCommand("section = Arch.makeSectionPlane(" + ss + ")")
        FreeCAD.ActiveDocument.commitTransaction()
        FreeCAD.ActiveDocument.recompute()
        if len(sel) == 1 and getattr(sel[0], "IfcClass", None) == "IfcProject":
            # remove the IFC project, otherwise we can't aggregate (circular loop)
            FreeCADGui.doCommand("section.Objects = []")
            # FreeCADGui.addModule("nativeifc.ifc_tools")
            # p = "FreeCAD.ActiveDocument."+sel[0].Name
            # FreeCADGui.doCommand("nativeifc.ifc_tools.aggregate(section,"+p+")")


FreeCADGui.addCommand("Arch_SectionPlane", Arch_SectionPlane())



# ***************************************************************************
# *   Copyright (c) 2019 Bernd Hahnebach <bernd@bimstatik.org>              *
# *   Copyright (c) 2024 Mario Passaglia <mpassaglia[at]cbc.uba.ar>         *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD FEM reinforced material"
__author__ = "Bernd Hahnebach, Mario Passaglia"
__url__ = "https://www.freecad.org"

## @package material_reinforced
#  \ingroup FEM
#  \brief reinforced object

from . import material_common
from .base_fempythonobject import _PropHelper


class MaterialReinforced(material_common.MaterialCommon):
    """
    The MaterialReinforced object
    """

    Type = "Fem::MaterialReinforced"

    def __init__(self, obj):
        super().__init__(obj)

        # overwrite Category enumeration
        obj.Category = ["Solid"]

    def _get_properties(self):
        prop = super()._get_properties()

        prop.append(
            _PropHelper(
                type="App::PropertyMap",
                name="Reinforcement",
                group="Composites",
                doc="Reinforcement material properties",
                value={},
            )
        )
        prop.append(
            _PropHelper(
                type="App::PropertyString",
                name="ReinforcementUUID",
                group="Composites",
                doc="Reinforcement material UUID",
                hidden=True,
                value="",
            )
        )

        return prop

    def onDocumentRestored(self, obj):
        super().onDocumentRestored(obj)

        # try update Reinforcement UUID from Reinforcement
        if not obj.ReinforcementUUID:
            obj.ReinforcementUUID = self._get_material_uuid(obj.Reinforcement)



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>        *
# *   Copyright (c) 2009, 2010 Ken Cline <cline@frii.com>                   *
# *   Copyright (c) 2020 Eliud Cabrera Castillo <e.cabrera-castillo@tum.de> *
# *   Copyright (c) 2025 The FreeCAD Project Association                    *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides functions to downgrade objects by different methods.

See also the `upgrade` function.
"""
## @package downgrade
# \ingroup draftfunctions
# \brief Provides functions to downgrade objects by different methods.

## \addtogroup draftfunctions
# @{
import FreeCAD as App
from draftmake import make_copy
from draftutils import utils
from draftutils import params
from draftutils import gui_utils
from draftutils.groups import is_group
from draftutils.messages import _msg
from draftutils.translate import translate


def downgrade(objects, delete=False, force=None):
    """Downgrade the given objects.

    This is a counterpart to `upgrade`.

    Parameters
    ----------
    objects: Part::Feature or list
        A single object to downgrade or a list
        containing various such objects.

    delete: bool, optional
        It defaults to `False`.
        If it is `True`, the old objects are deleted, and only the resulting
        object is kept.

    force: str, optional
        It defaults to `None`.
        Its value can be used to force a certain method of downgrading.
        It can be any of: `'explode'`, `'shapify'`, `'subtr'`, `'splitFaces'`,
        `'cut2'`, `'getWire'`, `'splitWires'`, or `'splitCompounds'`.

    Returns
    -------
    tuple
        A tuple containing two lists, a list of new objects
        and a list of objects to be deleted.

    See Also
    --------
    upgrade
    """

    # definitions of actions to perform

    def explode(obj):
        """Explode a Draft block or array."""
        obj_pl = obj.Placement
        # block:
        if getattr(obj, "Components", []):
            delete_list.append(obj)
            for comp in obj.Components:
                # Objects in Components are in the same parent group as the block.
                comp.Placement = obj_pl.multiply(comp.Placement)
                comp.Visibility = True
            return True
        # array:
        if getattr(obj, "Base", None) is None:
            return False
        if not hasattr(obj, "PlacementList"):
            return False
        # Array must be added to delete_list before base. See can_be_deleted.
        delete_list.append(obj)
        base = obj.Base
        if not base.Visibility:
            # Delete base if it is not visible. The can_be_deleted
            # function will check if it is not used elsewhere.
            delete_list.append(base)
        new_list = []
        if getattr(obj, "ExpandArray", False):
            delete_list.extend(obj.ElementList)
            for lnk in obj.ElementList:
                newobj = make_copy.make_copy(base)
                newobj.Placement = obj_pl.multiply(lnk.Placement)
                newobj.Visibility = True
                new_list.append(newobj)
        else:
            for arr_pl in obj.PlacementList:
                newobj = make_copy.make_copy(base)
                newobj.Placement = obj_pl.multiply(arr_pl)
                newobj.Visibility = True
                new_list.append(newobj)
        add_to_parent(obj, new_list)
        add_list.extend(new_list)
        return True

    def cut2(objects):
        """Cut 2nd object from 1st."""
        newobj = doc.addObject("Part::Cut", "Cut")
        newobj.Base = objects[0]
        newobj.Tool = objects[1]
        format(objects[0], [newobj])
        add_to_parent(objects[0], [newobj])
        add_list.append(newobj)
        return True

    def splitCompounds(objects):
        """Split solids contained in compound objects into new objects."""
        result = False
        for obj in objects:
            if not obj.Shape.Solids:
                continue
            new_list = []
            for solid in obj.Shape.Solids:
                newobj = doc.addObject("Part::Feature", "Solid")
                newobj.Shape = solid
                new_list.append(newobj)
            format(obj, new_list)
            add_to_parent(obj, new_list)
            add_list.extend(new_list)
            delete_list.append(obj)
            result = True
        return result

    def splitFaces(objects):
        """Split faces contained in objects into new objects."""
        result = False
        preserveFaceColor = params.get_param("preserveFaceColor")
        preserveFaceNames = params.get_param("preserveFaceNames")
        for obj in objects:
            if not obj.Shape.Faces:
                continue
            new_list = []
            if App.GuiUp and preserveFaceColor and obj.ViewObject:
                colors = obj.ViewObject.DiffuseColor
            else:
                colors = None
            label = getattr(obj, "Label", "")
            for ind, face in enumerate(obj.Shape.Faces):
                newobj = doc.addObject("Part::Feature", "Face")
                newobj.Shape = face
                format(obj, [newobj])
                if App.GuiUp and preserveFaceColor and colors:
                    # At this point, some single-color objects might have just
                    # a single value in colors for all faces, so we handle that:
                    if ind < len(colors):
                        color = colors[ind]
                    else:
                        color = colors[0]
                    newobj.ViewObject.ShapeColor = color
                if preserveFaceNames:
                    newobj.Label = "{} {}".format(label, newobj.Label)
                new_list.append(newobj)
            add_to_parent(obj, new_list)
            add_list.extend(new_list)
            delete_list.append(obj)
            result = True
        return result

    def subtr(objects):
        """Subtract faces from the first one."""
        faces = []
        done_list = []
        for obj in objects:
            if obj.Shape.Faces:
                faces.extend(obj.Shape.Faces)
                done_list.append(obj)
        if not faces:
            return False
        main_face = faces.pop(0)
        for face in faces:
            main_face = main_face.cut(face)
        if main_face.isNull():
            return False
        newobj = doc.addObject("Part::Feature", "Subtraction")
        newobj.Shape = main_face
        format(done_list[0], [newobj])
        add_to_parent(done_list[0], [newobj])
        add_list.append(newobj)
        delete_list.extend(done_list)
        return True

    def getWire(obj):
        """Get the wire from a face object."""
        if not obj.Shape.Faces:
            return False
        new_list = []
        for wire in obj.Shape.Faces[0].Wires:
            newobj = doc.addObject("Part::Feature", "Wire")
            newobj.Shape = wire
            new_list.append(newobj)
        format(obj, new_list)
        add_to_parent(obj, new_list)
        add_list.extend(new_list)
        delete_list.append(obj)
        return True

    def splitWires(objects):
        """Split the wires contained in objects into edges."""
        result = False
        for obj in objects:
            if not obj.Shape.Edges:
                continue
            new_list = []
            for edge in obj.Shape.Edges:
                newobj = doc.addObject("Part::Feature", "Edge")
                newobj.Shape = edge
                new_list.append(newobj)
            format(obj, new_list)
            add_to_parent(obj, new_list)
            add_list.extend(new_list)
            delete_list.append(obj)
            result = True
        return result

    def _shapify(obj):
        """Wrapper for utils.shapify."""
        newobj = utils.shapify(obj, delete=False)
        if newobj:
            format(obj, [newobj])
            add_to_parent(obj, [newobj])
            add_list.append(newobj)
            delete_list.append(obj)
            return True
        return False

    # helper functions (same as in upgrade.py)

    def get_parent(obj):
        # Problem with obj.getParent():
        # https://github.com/FreeCAD/FreeCAD/issues/19600
        parent = obj.getParentGroup()
        if parent is not None:
            return parent
        return obj.getParentGeoFeatureGroup()

    def can_be_deleted(obj):
        if not obj.InList:
            return True
        for other in obj.InList:
            if is_group(other):
                continue
            if other.TypeId == "App::Part":
                continue
            return False
        return True

    def delete_object(obj):
        if utils.is_deleted(obj):
            return
        parent = get_parent(obj)
        if parent is not None and parent.TypeId == "PartDesign::Body":
            obj = parent
        if not can_be_deleted(obj):
            # Make obj invisible instead:
            obj.Visibility = False
            return
        if obj.TypeId == "PartDesign::Body":
            obj.removeObjectsFromDocument()
        doc.removeObject(obj.Name)

    def add_to_parent(obj, new_list):
        parent = get_parent(obj)
        if parent is None:
            if doc.getObject("Draft_Construction"):
                # This cludge is required because the make_* commands may
                # put new objects in the construction group.
                constr_group = doc.getObject("Draft_Construction")
                for newobj in new_list:
                    constr_group.removeObject(newobj)
            return
        if parent.TypeId == "PartDesign::Body":
            # We don't add to a PD Body. We process its placement and
            # add to its parent instead.
            for newobj in new_list:
                newobj.Placement = parent.Placement.multiply(newobj.Placement)
            add_to_parent(parent, new_list)
            return
        for newobj in new_list:
            # Using addObject is different from just changing the Group property.
            # With addObject the object will be added to the parent group, but if
            # that is a normal group, also to that group's parent GeoFeatureGroup,
            # if available.
            parent.addObject(newobj)

    def format(obj, new_list):
        for newobj in new_list:
            gui_utils.format_object(newobj, obj, ignore_construction=True)

    doc = App.ActiveDocument
    add_list = []
    delete_list = []
    result = False

    if not isinstance(objects, list):
        objects = [objects]
    if not objects:
        return add_list, delete_list

    # analyzing objects
    solids = []
    faces = []
    edges = []
    onlyedges = True
    parts = []

    for obj in objects:
        if hasattr(obj, "Shape"):
            for solid in obj.Shape.Solids:
                solids.append(False)
            for face in obj.Shape.Faces:
                faces.append(face)
            for edge in obj.Shape.Edges:
                edges.append(edge)
            if obj.Shape.ShapeType != "Edge":
                onlyedges = False
            parts.append(obj)
    objects = parts

    if not objects:
        result = False

    elif force:
        # functions that work on a single object:
        single_funcs = {"explode": explode, "getWire": getWire, "shapify": _shapify}
        # functions that work on multiple objects:
        multi_funcs = {
            "cut2": cut2,
            "splitCompounds": splitCompounds,
            "splitFaces": splitFaces,
            "splitWires": splitWires,
            "subtr": subtr,
        }
        if force in single_funcs:
            result = any([single_funcs[force](obj) for obj in objects])
        elif force in multi_funcs:
            result = multi_funcs[force](objects)
        else:
            _msg(translate("draft", "Downgrade: Unknown force method:") + " " + force)
            result = False

    else:
        parent = get_parent(objects[0])
        same_parent = True
        same_parent_type = getattr(parent, "TypeId", "")  # "" for global space.
        if len(objects) > 1:
            for obj in objects[1:]:
                if get_parent(obj) != parent:
                    same_parent = False
                    same_parent_type = None
                    break

        # we have a block, we explode it
        if len(objects) == 1 and utils.get_type(objects[0]) == "Block":
            result = explode(objects[0])
            if result:
                _msg(translate("draft", "Found 1 block: exploding it"))

        # we have an array, we explode it
        elif len(objects) == 1 and "Array" in utils.get_type(objects[0]):
            result = explode(objects[0])
            if result:
                _msg(translate("draft", "Found 1 array: exploding it"))

        # special case, we have one parametric object: we "de-parametrize" it
        elif (
            len(objects) == 1
            and hasattr(objects[0], "Shape")
            and (
                hasattr(objects[0], "Base")
                or hasattr(objects[0], "Profile")
                or hasattr(objects[0], "Sections")
            )
        ):
            result = _shapify(objects[0])
            if result:
                _msg(translate("draft", "Found 1 parametric object: breaking its dependencies"))

        # we have one multi-solids compound object: extract its solids
        elif len(objects) == 1 and hasattr(objects[0], "Shape") and len(solids) > 1:
            result = splitCompounds(objects)
            if result:
                _msg(translate("draft", "Found 1 multi-solids compound: exploding it"))

        # we have only 2 objects: cut 2nd from 1st
        elif len(objects) == 2 and same_parent and same_parent_type != "PartDesign::Body":
            result = cut2(objects)
            if result:
                _msg(translate("draft", "Found 2 objects: subtracting them"))

        elif len(faces) > 1:
            # one object with several faces: split it
            if len(objects) == 1:
                result = splitFaces(objects)
                if result:
                    _msg(translate("draft", "Found several faces: splitting them"))
            # several objects: remove all the faces from the first one
            elif same_parent and same_parent_type != "PartDesign::Body":
                result = subtr(objects)
                if result:
                    _msg(
                        translate(
                            "draft", "Found several faces: subtracting them from the first one"
                        )
                    )

        # only one face: we extract its wires
        elif len(faces) > 0:
            result = getWire(objects[0])
            if result:
                _msg(translate("draft", "Found 1 face: extracting its wires"))

        # no faces: split wire into single edges
        elif not onlyedges:
            result = splitWires(objects)
            if result:
                _msg(translate("draft", "Found only wires: extracting their edges"))

    # no result has been obtained
    if not result:
        _msg(translate("draft", "Unable to downgrade these objects"))

    if delete:
        for obj in delete_list:
            delete_object(obj)

    gui_utils.select(add_list)
    return add_list, delete_list


## @}



# SPDX-License-Identifier: LGPL-2.1-or-later

#***************************************************************************
#*   Copyright (c) 2002 Juergen Riegel <juergen.riegel@web.de>             *
#*                                                                         *
#*   This file is part of the FreeCAD CAx development system.              *
#*                                                                         *
#*   This program is free software; you can redistribute it and/or modify  *
#*   it under the terms of the GNU Lesser General Public License (LGPL)    *
#*   as published by the Free Software Foundation; either version 2 of     *
#*   the License, or (at your option) any later version.                   *
#*   for detail see the LICENCE text file.                                 *
#*                                                                         *
#*   FreeCAD is distributed in the hope that it will be useful,            *
#*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
#*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
#*   GNU Lesser General Public License for more details.                   *
#*                                                                         *
#*   You should have received a copy of the GNU Library General Public     *
#*   License along with FreeCAD; if not, write to the Free Software        *
#*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
#*   USA                                                                   *
#*                                                                         *
#***************************************************************************/

# OpenSCAD gui init module
#
# Gathering all the information to start FreeCAD
# This is the second one of three init scripts, the third one
# runs when the gui is up

import FreeCAD
import sys


class OpenSCADWorkbench(Workbench):
    "OpenSCAD workbench object"

    def __init__(self):
        self.__class__.Icon = (
            FreeCAD.getResourceDir()
            + "Mod/OpenSCAD/Resources/icons/OpenSCADWorkbench.svg"
        )
        self.__class__.MenuText = "OpenSCAD"
        self.__class__.ToolTip = (
            "OpenSCAD is an application for creating solid 3D CAD.\n"
            "FreeCAD utizes OpenSCAD's capability as a script-only based modeller that uses its own description language\n"
            "Note: the Mesh workbench heavily uses the boolean operations of this workbench because they are quite robust"
        )

    def Initialize(self):
        def QT_TRANSLATE_NOOP(scope, text):
            return text

        import OpenSCAD_rc, OpenSCADCommands

        commands = [
            "OpenSCAD_ReplaceObject",
            "OpenSCAD_RemoveSubtree",
            "OpenSCAD_RefineShapeFeature",
            "OpenSCAD_MirrorMeshFeature",
            "OpenSCAD_ScaleMeshFeature",
            "OpenSCAD_ResizeMeshFeature",
            "OpenSCAD_IncreaseToleranceFeature",
            "OpenSCAD_Edgestofaces",
            "OpenSCAD_ExpandPlacements",
            "OpenSCAD_ExplodeGroup",
        ]
        toolbarcommands = [
            "OpenSCAD_ReplaceObject",
            "OpenSCAD_RemoveSubtree",
            "OpenSCAD_ExplodeGroup",
            "OpenSCAD_RefineShapeFeature",
            "OpenSCAD_IncreaseToleranceFeature",
        ]
        import PartGui

        parttoolbarcommands = [
            "Part_CheckGeometry",
            "Part_Primitives",
            "Part_Builder",
            "Part_Cut",
            "Part_Fuse",
            "Part_Common",
            "Part_Extrude",
            "Part_Revolve",
        ]
        import FreeCAD
        translate = FreeCAD.Qt.translate

        param = FreeCAD.ParamGet("User parameter:BaseApp/Preferences/Mod/OpenSCAD")
        openscadfilename = param.GetString("openscadexecutable")
        if not openscadfilename:
            import OpenSCADUtils

            openscadfilename = OpenSCADUtils.searchforopenscadexe()
            if openscadfilename:  # automatic search was successful
                FreeCAD.addImportType("OpenSCAD Format (*.scad *.SCAD)", "importCSG")
                param.SetString(
                    "openscadexecutable", openscadfilename
                )  # save the result
        if openscadfilename:
            commands.extend(
                [
                    "OpenSCAD_AddOpenSCADElement",
                    "OpenSCAD_MeshBoolean",
                    "OpenSCAD_Hull",
                    "OpenSCAD_Minkowski",
                ]
            )

            toolbarcommands.extend(
                [
                    "OpenSCAD_AddOpenSCADElement",
                    "OpenSCAD_MeshBoolean",
                    "OpenSCAD_Hull",
                    "OpenSCAD_Minkowski",
                ]
            )
        else:
            FreeCAD.Console.PrintWarning("OpenSCAD executable not found\n")

        transferMechanism = param.GetInt("transfermechanism", 0)
        if openscadfilename and transferMechanism == 0:
            # We are using the Python temp-directory creation function
            if "snap" in openscadfilename:
                FreeCAD.Console.PrintMessage(
                    translate(
                        "OpenSCAD",
                        "It looks like you may be using a Snap version of OpenSCAD.",
                    )
                    + " "
                    + translate(
                        "OpenSCAD",
                        "If OpenSCAD execution fails to load the temporary file, use FreeCAD's OpenSCAD Workbench Preferences to change the transfer mechanism.",
                    )
                    + "\n"
                )
            elif sys.executable.startswith("/tmp/"):  # Heuristic for AppImages
                FreeCAD.Console.PrintMessage(
                    translate(
                        "OpenSCAD",
                        "It looks like you may be using a sandboxed version of FreeCAD.",
                    )
                    + " "
                    + translate(
                        "OpenSCAD",
                        "If OpenSCAD execution fails to load the temporary file, use FreeCAD's OpenSCAD Workbench Preferences to change the transfer mechanism.",
                    )
                    + "\n"
                )

        self.appendToolbar(
            QT_TRANSLATE_NOOP("Workbench", "OpenSCAD Tools"), toolbarcommands
        )
        self.appendMenu("OpenSCAD", commands)
        self.appendToolbar(
            QT_TRANSLATE_NOOP("Workbench", "Frequently-used Part WB tools"), parttoolbarcommands
        )

        FreeCADGui.addIconPath(":/icons")
        FreeCADGui.addLanguagePath(":/translations")
        FreeCADGui.addPreferencePage(":/ui/openscadprefs-base.ui", "OpenSCAD")

    def GetClassName(self):
        return "Gui::PythonWorkbench"


Gui.addWorkbench(OpenSCADWorkbench())

# Not all of the GUI tests will require an OpenSCAD binary (CSG import and export don't)
FreeCAD.__unit_test__ += ["TestOpenSCADGui"]



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2019 Zheng, Lei (realthunder)<realthunder.dev@gmail.com>*
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides the base class for Link objects used by other objects.

This class was created by realthunder during the `LinkMerge`
to demonstrate how to use the `App::Link` objects to create
Link aware arrays.
It is used by `draftobject.array` (ortho, polar, circular)
and `draftobject.patharray` to create respective Link arrays.

NOTE: this class is a bit mysterious. We need more documentation
on how the properties are being set, and how the code interacts with
the arrays that use it.
"""
## @package draftlink
# \ingroup draftobjects
# \brief Provides the base class for Link objects used by other objects.

import lazy_loader.lazy_loader as lz
from PySide.QtCore import QT_TRANSLATE_NOOP

import FreeCAD as App
from draftutils import gui_utils
from draftutils.messages import _log

from draftobjects.base import DraftObject

# Delay import of module until first use because it is heavy
Part = lz.LazyLoader("Part", globals(), "Part")
DraftGeomUtils = lz.LazyLoader("DraftGeomUtils", globals(), "DraftGeomUtils")

## \addtogroup draftobjects
# @{


class DraftLink(DraftObject):
    """New class to use the App::Link objects in arrays.

    Introduced by realthunder.
    This is subclassed by `draftobjects.array.Array`
    and by `draftobjects.patharray.PathArray`.
    """

    def __init__(self, obj, tp):
        self.use_link = False if obj else True
        super().__init__(obj, tp)
        if obj:
            self.attach(obj)

    def dumps(self):
        """Return a tuple of all serializable objects or None."""
        return self.__dict__

    def loads(self, state):
        """Set some internal properties for all restored objects."""
        if isinstance(state, dict):
            self.__dict__ = state
        else:
            self.use_link = False
            super().loads(state)

    def attach(self, obj):
        """Set up the properties when the object is attached."""
        if self.use_link:
            obj.addExtension("App::LinkExtensionPython")
            self.linkSetup(obj)

    def canLinkProperties(self, _obj):
        """Link properties.

        TODO: add more explanation. Overrides a C++ method?
        """
        return False

    def linkSetup(self, obj):
        """Set up the link properties on attachment."""
        obj.configLinkProperty("Placement", LinkedObject="Base")

        if not hasattr(obj, "AlwaysSyncPlacement"):
            _tip = QT_TRANSLATE_NOOP(
                "App::Property",
                "Force sync pattern placements even when array elements are expanded",
            )
            obj.addProperty("App::PropertyBool", "AlwaysSyncPlacement", "Draft", _tip, locked=True)

        if hasattr(obj, "ShowElement"):
            # Rename 'ShowElement' property to 'ExpandArray' to avoid conflict
            # with native App::Link
            obj.configLinkProperty("ShowElement")
            showElement = obj.ShowElement

            _tip = QT_TRANSLATE_NOOP("App::Property", "Show the individual array elements")
            obj.addProperty("App::PropertyBool", "ExpandArray", "Draft", _tip, locked=True)

            obj.ExpandArray = showElement
            obj.configLinkProperty(ShowElement="ExpandArray")
            obj.removeProperty("ShowElement")
        else:
            obj.configLinkProperty(ShowElement="ExpandArray")

        if getattr(obj, "ExpandArray", False):
            obj.setPropertyStatus("PlacementList", "Immutable")
        else:
            obj.setPropertyStatus("PlacementList", "-Immutable")

        if not hasattr(obj, "LinkTransform"):
            obj.addProperty("App::PropertyBool", "LinkTransform", " Link", locked=True)

        if not hasattr(obj, "ColoredElements"):
            obj.addProperty("App::PropertyLinkSubHidden", "ColoredElements", " Link", locked=True)
            obj.setPropertyStatus("ColoredElements", "Hidden")

        if not hasattr(obj, "LinkCopyOnChange"):
            obj.addProperty("App::PropertyEnumeration", "LinkCopyOnChange", " Link", locked=True)

        obj.configLinkProperty("LinkCopyOnChange", "LinkTransform", "ColoredElements")

        if not getattr(obj, "Fuse", False):
            obj.setPropertyStatus("Shape", "Transient")

    def getViewProviderName(self, _obj):
        """Override the view provider name."""
        if self.use_link:
            return "Gui::ViewProviderLinkPython"
        return ""

    def migrate_attributes(self, obj):
        """Migrate old attribute names to new names if they exist.

        This is done to comply with Python guidelines or fix small issues
        in older code.
        """
        if hasattr(self, "useLink"):
            # This is only needed for some models created in 0.19
            # while it was in development. Afterwards,
            # all models should use 'use_link' by default
            # and this won't be run.
            self.use_link = bool(self.useLink)
            _log("v0.19, {}, 'useLink' will be migrated to 'use_link'".format(obj.Name))
            del self.useLink

    def onDocumentRestored(self, obj):
        """Execute code when the document in restored."""
        self.migrate_attributes(obj)

        if self.use_link:
            self.linkSetup(obj)
        else:
            obj.setPropertyStatus("Shape", "-Transient")

        if obj.Shape.isNull():
            if getattr(obj, "PlacementList", None):
                self.buildShape(obj, obj.Placement, obj.PlacementList)
            else:
                self.execute(obj)

        super().onDocumentRestored(obj)
        if hasattr(obj, "LinkTransform"):
            gui_utils.restore_view_object(
                obj, vp_module="view_draftlink", vp_class="ViewProviderDraftLink", format=False
            )
        else:
            gui_utils.restore_view_object(
                obj, vp_module="view_array", vp_class="ViewProviderDraftArray"
            )

    def buildShape(self, obj, pl, pls):
        """Build the shape of the link object."""
        if self.use_link:
            if not getattr(obj, "ExpandArray", False) or obj.Count != len(pls):
                obj.setPropertyStatus("PlacementList", "-Immutable")
                obj.PlacementList = pls
                obj.setPropertyStatus("PlacementList", "Immutable")
                obj.Count = len(pls)
            if getattr(obj, "ExpandArray", False) and getattr(obj, "AlwaysSyncPlacement", False):
                for pla, child in zip(pls, obj.ElementList):
                    child.Placement = pla
        else:
            obj.PlacementList = pls
            if obj.Count != len(pls):
                obj.Count = len(pls)

        if obj.Base:
            shape = getattr(obj.Base, "Shape", None)
            if not isinstance(shape, Part.Shape):
                obj.Shape = Part.Shape()
            elif shape.isNull():
                _err_msg = "'{}' cannot build shape " "from '{}'\n".format(
                    obj.Label, obj.Base.Label
                )
                raise RuntimeError(_err_msg)
            else:
                # Resetting the Placement of the copied shape does not work for
                # Part_Vertex and Draft_Point objects, we need to transform:
                place = shape.Placement
                shape = shape.copy()
                shape.transformShape(place.Matrix.inverse())
                base = []
                for i, pla in enumerate(pls):
                    vis = getattr(obj, "VisibilityList", [])
                    if len(vis) > i and not vis[i]:
                        continue

                    base.append(shape.transformed(pla.toMatrix()))

                if getattr(obj, "Fuse", False) and len(base) > 1:
                    obj.Shape = base[0].multiFuse(base[1:]).removeSplitter()
                else:
                    obj.Shape = Part.makeCompound(base)

                if not DraftGeomUtils.isNull(pl):
                    obj.Placement = pl

        if self.use_link:
            return False  # return False to call LinkExtension::execute()

    def onChanged(self, obj, prop):
        """Execute when a property changes."""
        self.props_changed_store(prop)

        if not getattr(self, "use_link", False):
            return

        if prop == "Fuse":
            if obj.Fuse:
                obj.setPropertyStatus("Shape", "-Transient")
            else:
                obj.setPropertyStatus("Shape", "Transient")
        elif prop == "ExpandArray":
            if hasattr(obj, "PlacementList"):
                if obj.ExpandArray:
                    obj.setPropertyStatus("PlacementList", "-Immutable")
                else:
                    obj.setPropertyStatus("PlacementList", "Immutable")

    def getPlacementOf(self, fp, subname, targetObj=None):
        """
        Return the placement of the sub-object relative to the link object.
        """

        # _getShowElementValue is mapped to ExpandArray.
        if getattr(fp, "ExpandArray", False):
            # Return None to fall back to the C++ implementation
            return None

        # We start with the object's own placement.
        plc = fp.Placement

        if not subname:
            return plc

        names = subname.split(".")

        linked_obj = getattr(fp, "Base", None)
        if not names or fp == targetObj or not linked_obj:
            return plc

        doc = linked_obj.Document

        try:
            i = int(names[0])
        except ValueError:
            return None

        # Check bounds in PlacementList
        plcs = fp.PlacementList
        if i < 0 or i >= len(plcs):
            return plc

        # Accumulate the element placement
        plc = plc * plcs[i]

        if len(names) < 2:
            return plc

        subObj = doc.getObject(names[1])
        if not subObj:
            return plc

        newSub = ".".join(names[2:])

        return plc * subObj.getPlacementOf(newSub, targetObj)


# Alias for compatibility with old versions of v0.19
_DraftLink = DraftLink

## @}



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2017 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************

"""The BIM Ungroup command"""

import FreeCAD
import FreeCADGui

QT_TRANSLATE_NOOP = FreeCAD.Qt.QT_TRANSLATE_NOOP


class BIM_Ungroup:

    def GetResources(self):
        return {
            "Pixmap": "Draft_AddToGroup",
            "MenuText": QT_TRANSLATE_NOOP("BIM_Convert", "Remove From Group"),
            "ToolTip": QT_TRANSLATE_NOOP(
                "BIM_Convert", "Removes this object from its parent group"
            ),
        }

    def Activated(self):
        sel = FreeCADGui.Selection.getSelection()
        first = True
        if sel:
            for obj in sel:
                for parent in obj.InList:
                    if parent.isDerivedFrom("App::DocumentObjectGroup") or parent.hasExtension(
                        "App::GroupExtension"
                    ):
                        if obj in parent.Group:
                            if first:
                                FreeCAD.ActiveDocument.openTransaction("Ungroup")
                                first = False
                            if hasattr(parent, "removeObject"):
                                parent.removeObject(obj)
                            else:
                                g = parent.Group
                                g.remove(obj)
                                parent.Group = g
        if not first:
            FreeCAD.ActiveDocument.commitTransaction()
            FreeCAD.ActiveDocument.recompute()


FreeCADGui.addCommand("BIM_Ungroup", BIM_Ungroup())



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2009, 2010 Yorik van Havre <yorik@uncreated.net>        *
# *   Copyright (c) 2009, 2010 Ken Cline <cline@frii.com>                   *
# *   Copyright (c) 2020 FreeCAD Developers                                 *
# *   Copyright (c) 2025 FreeCAD Project Association                        *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************
"""Provides the viewprovider code for the Clone object."""
## @package view_clone
# \ingroup draftviewproviders
# \brief Provides the viewprovider code for the Clone object.

## \addtogroup draftviewproviders
# @{
from PySide import QtCore
from PySide import QtGui

import FreeCADGui as Gui
from drafttaskpanels import task_scale
from draftutils.translate import translate


class ViewProviderClone:
    """a view provider that displays a Clone icon instead of a Draft icon"""

    def __init__(self, vobj):
        vobj.Proxy = self

    def attach(self, vobj):
        self.Object = vobj.Object
        return

    def getIcon(self):
        return ":/icons/Draft_Clone.svg"

    def setEdit(self, vobj, mode):
        if mode != 0:
            return None

        self.task = task_scale.ScaleTaskPanelEdit(self.Object)
        Gui.Control.showDialog(self.task)
        return True

    def unsetEdit(self, vobj, mode):
        if mode != 0:
            return None

        self.task.finish()
        return True

    def setupContextMenu(self, vobj, menu):
        action_edit = QtGui.QAction(translate("draft", "Edit"), menu)
        QtCore.QObject.connect(action_edit, QtCore.SIGNAL("triggered()"), self.edit)
        menu.addAction(action_edit)

    def edit(self):
        Gui.ActiveDocument.setEdit(self.Object, 0)

    def dumps(self):
        return None

    def loads(self, state):
        return None

    def getDisplayModes(self, vobj):
        return []

    def setDisplayMode(self, mode):
        return mode


# Alias for compatibility with v0.18 and earlier
_ViewProviderClone = ViewProviderClone

## @}



# SPDX-License-Identifier: LGPL-2.1-or-later
# /****************************************************************************
#                                                                           *
#    Copyright (c) 2025 Weston Schmidt <weston_schmidt@alumni.purdue.edu>   *
#                                                                           *
#    This file is part of FreeCAD.                                          *
#                                                                           *
#    FreeCAD is free software: you can redistribute it and/or modify it     *
#    under the terms of the GNU Lesser General Public License as            *
#    published by the Free Software Foundation, either version 2.1 of the   *
#    License, or (at your option) any later version.                        *
#                                                                           *
#    FreeCAD is distributed in the hope that it will be useful, but         *
#    WITHOUT ANY WARRANTY; without even the implied warranty of             *
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU       *
#    Lesser General Public License for more details.                        *
#                                                                           *
#    You should have received a copy of the GNU Lesser General Public       *
#    License along with FreeCAD. If not, see                                *
#    <https://www.gnu.org/licenses/>.                                       *
#                                                                           *
# ***************************************************************************/

"""
Mock classes for FreeCAD GUI testing.

This module provides mock implementations of FreeCAD and Qt classes
to enable unit testing without requiring the full FreeCAD environment.
"""

# pylint: disable=too-few-public-methods

from unittest.mock import MagicMock


def create_mock_qicon():
    """Create a mock QIcon with fromTheme static method."""
    mock_qicon = MagicMock()
    mock_qicon.fromTheme = MagicMock(return_value=mock_qicon)
    return mock_qicon


def create_mock_qtreewidgetitem():
    """Create a mock QTreeWidgetItem with required methods and state tracking."""
    mock_item = MagicMock()

    # Add state tracking for setText/text functionality
    mock_item.text_values = {}
    mock_item.data_values = {}
    mock_item.children = []

    def mock_set_text(column, text):
        mock_item.text_values[column] = text

    def mock_get_text(column):
        return mock_item.text_values.get(column, "")

    def mock_set_data(column, role, data):
        mock_item.data_values[(column, role)] = data

    def mock_get_data(column, role):
        return mock_item.data_values.get((column, role))

    def mock_child_count():
        return len(mock_item.children)

    def mock_child(index):
        if 0 <= index < len(mock_item.children):
            return mock_item.children[index]
        return None

    def mock_add_child(child):
        mock_item.children.append(child)

    # Configure the mock with specific behaviors
    mock_item.setText = mock_set_text
    mock_item.text = mock_get_text
    mock_item.setData = mock_set_data
    mock_item.data = mock_get_data
    mock_item.childCount = mock_child_count
    mock_item.child = mock_child
    mock_item.addChild = mock_add_child

    return mock_item


def create_mock_checkbox():
    """Create a mock CheckBox with state tracking."""
    mock_checkbox = MagicMock()
    mock_checkbox.checked = False

    def mock_set_checked(checked):
        mock_checkbox.checked = checked

    def mock_is_checked():
        return mock_checkbox.checked

    mock_checkbox.setChecked = mock_set_checked
    mock_checkbox.isChecked = mock_is_checked
    mock_checkbox.stateChanged = MagicMock()

    return mock_checkbox


def create_mock_line_edit():
    """Create a mock LineEdit."""
    mock_line_edit = MagicMock()
    mock_line_edit.text.return_value = ""
    mock_line_edit.textChanged = MagicMock()
    return mock_line_edit


def create_mock_part_list():
    """Create a mock PartList with required functionality."""
    mock_part_list = MagicMock()
    mock_part_list.items = []

    def mock_clear():
        mock_part_list.items = []

    def mock_add_top_level_item(item):
        mock_part_list.items.append(item)

    def mock_top_level_item_count():
        return len(mock_part_list.items)

    def mock_top_level_item(index):
        if 0 <= index < len(mock_part_list.items):
            return mock_part_list.items[index]
        return None

    mock_part_list.clear = mock_clear
    mock_part_list.addTopLevelItem = mock_add_top_level_item
    mock_part_list.topLevelItemCount = mock_top_level_item_count
    mock_part_list.topLevelItem = mock_top_level_item
    mock_part_list.sizeHintForRow.return_value = 20

    # Add other required attributes/methods
    mock_part_list.itemClicked = MagicMock()
    mock_part_list.itemDoubleClicked = MagicMock()
    mock_part_list.header.return_value = MagicMock()

    return mock_part_list


def create_mock_form():
    """Create a mock Form with all required components."""
    mock_form = MagicMock()
    mock_form.partList = create_mock_part_list()
    mock_form.CheckBox_ShowOnlyParts = create_mock_checkbox()
    mock_form.CheckBox_RigidSubAsm = create_mock_checkbox()
    mock_form.openFileButton = MagicMock()
    mock_form.openFileButton.clicked = MagicMock()
    mock_form.filterPartList = create_mock_line_edit()
    return mock_form


def create_mock_pyside_uic():
    """Create a mock PySideUic with loadUi method."""
    mock_uic = MagicMock()
    mock_uic.loadUi.return_value = create_mock_form()
    return mock_uic


def create_mock_gui_document(doc_name):
    """Create a mock GUI document."""
    mock_doc = MagicMock()
    mock_doc.Name = doc_name
    mock_doc.getObject.return_value = None
    mock_doc.TreeRootObjects = []
    return mock_doc


# Factory functions for creating specific mock instances
MockQIcon = create_mock_qicon
MockQTreeWidgetItem = create_mock_qtreewidgetitem
MockPySideUic = create_mock_pyside_uic
MockGetDocument = create_mock_gui_document
MockAddModule = MagicMock()
MockDoCommandSkip = MagicMock()



# SPDX-License-Identifier: LGPL-2.1-or-later
# SPDX-FileCopyrightText: 2022 FreeCAD Project Association
# SPDX-FileNotice: Part of the AddonManager.

################################################################################
#                                                                              #
#   This addon is free software: you can redistribute it and/or modify         #
#   it under the terms of the GNU Lesser General Public License as             #
#   published by the Free Software Foundation, either version 2.1              #
#   of the License, or (at your option) any later version.                     #
#                                                                              #
#   This addon is distributed in the hope that it will be useful,              #
#   but WITHOUT ANY WARRANTY; without even the implied warranty                #
#   of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.                    #
#   See the GNU Lesser General Public License for more details.                #
#                                                                              #
#   You should have received a copy of the GNU Lesser General Public           #
#   License along with this addon. If not, see https://www.gnu.org/licenses    #
#                                                                              #
################################################################################

"""Provides the PackageDetails widget."""

import os

from PySideWrapper import QtCore, QtWidgets

import addonmanager_freecad_interface as fci

from addonmanager_metadata import (
    get_branch_from_metadata,
    get_repo_url_from_metadata,
)
from addonmanager_workers_startup import CheckSingleUpdateWorker
from addonmanager_git import GitManager, NoGitFound
from Addon import Addon
from addonmanager_readme_controller import ReadmeController
from Widgets.addonmanager_widget_package_details_view import UpdateInformation, WarningFlags

translate = fci.translate


class PackageDetailsController(QtCore.QObject):
    """Manages the display of the package README information."""

    back = QtCore.Signal()
    install = QtCore.Signal(Addon)
    uninstall = QtCore.Signal(Addon)
    update = QtCore.Signal(Addon)
    execute = QtCore.Signal(Addon)
    update_status = QtCore.Signal(Addon)

    def __init__(self, widget=None):
        super().__init__()
        self.ui = widget
        self.readme_controller = ReadmeController(self.ui.readme_browser)
        self.worker = None
        self.addon = None
        self.update_check_thread = None
        self.original_disabled_state = None
        self.original_status = None
        self.check_for_update_worker = None
        try:
            self.git_manager = GitManager()
        except NoGitFound:
            self.git_manager = None

        self.ui.button_bar.back.clicked.connect(self.back.emit)
        self.ui.button_bar.run_macro.clicked.connect(lambda: self.execute.emit(self.addon))
        self.ui.button_bar.install.clicked.connect(lambda: self.install.emit(self.addon))
        self.ui.button_bar.uninstall.clicked.connect(lambda: self.uninstall.emit(self.addon))
        self.ui.button_bar.update.clicked.connect(lambda: self.update.emit(self.addon))
        self.ui.button_bar.enable.clicked.connect(self.enable_clicked)
        self.ui.button_bar.disable.clicked.connect(self.disable_clicked)
        self.ui.button_bar.install_branch.connect(self.install_branch)

    def show_addon(self, addon: Addon) -> None:
        """The main entry point for this class shows the package details and related buttons
        for the provided repo."""
        self.addon = addon
        self.readme_controller.set_addon(addon)
        self.original_disabled_state = self.addon.is_disabled()
        if addon is not None:
            self.ui.button_bar.show()
            if addon.repo_type == Addon.Kind.MACRO:
                self.set_up_macro_display()
            else:
                self.set_up_non_macro_display()
            self.set_up_updater()
        else:
            self.ui.button_bar.hide()

    def set_up_updater(self):
        if self.worker is not None:
            if not self.worker.isFinished():
                self.worker.requestInterruption()
                self.worker.wait()

        self.ui.button_bar.set_update_check_status(False)

        installed = self.addon.status() != Addon.Status.NOT_INSTALLED
        self.ui.set_installed(installed)
        if self.addon.metadata is not None:
            self.ui.set_url(get_repo_url_from_metadata(self.addon.metadata))
        else:
            self.ui.set_url(None)  # to reset it and hide it
        update_info = UpdateInformation()
        if installed:
            update_info.unchecked = self.addon.status() == Addon.Status.UNCHECKED
            update_info.update_available = self.addon.status() == Addon.Status.UPDATE_AVAILABLE
            update_info.check_in_progress = False  # TODO: Implement the "check in progress" status
            if self.addon.metadata:
                update_info.branch = get_branch_from_metadata(self.addon.metadata)
                update_info.version = str(self.addon.metadata.version)
            elif self.addon.macro:
                update_info.version = str(self.addon.macro.version)
            self.ui.set_update_available(update_info)
            self.ui.set_location(
                self.addon.macro_directory
                if self.addon.repo_type == Addon.Kind.MACRO
                else os.path.join(self.addon.mod_directory, self.addon.name)
            )

        if self.addon.status() == Addon.Status.UNCHECKED:
            # If the user is looking at this addon, and it has not been checked for updates (e.g.,
            # it is a custom addon that has to directly query its git repo), then do that check now.
            if not self.update_check_thread:
                self.update_check_thread = QtCore.QThread()
                self.update_check_thread.setObjectName(
                    "PackageDetailsController update check thread"
                )
            self.check_for_update_worker = CheckSingleUpdateWorker(self.addon)
            self.check_for_update_worker.moveToThread(self.update_check_thread)
            self.update_check_thread.finished.connect(self.check_for_update_worker.deleteLater)
            self.update_check_thread.started.connect(self.check_for_update_worker.do_work)
            self.check_for_update_worker.update_status.connect(self.display_repo_status)
            self.update_check_thread.start()
            if self.update_check_thread.isRunning():
                self.ui.button_bar.set_update_check_status(True)

        flags = WarningFlags()
        self.ui.set_warning_flags(flags)

    def set_up_non_macro_display(self):
        branches = []
        if self.addon.status() == Addon.Status.NOT_INSTALLED:
            branches.append(self.addon.branch_display_name)
        if self.addon.sub_addons:
            branches.extend(self.addon.sub_addons.keys())
        self.ui.button_bar.set_installation_status(
            installed=self.addon.status() != Addon.Status.NOT_INSTALLED,
            available_branches=branches,
            disabled=self.addon.is_disabled(),
        )
        self.ui.button_bar.set_can_run(False)
        self.ui.button_bar.set_update_available(
            self.addon.status() == Addon.Status.UPDATE_AVAILABLE
        )
        if self.addon.name == "AddonManager":
            self.ui.button_bar.setup_for_addon_manager()  # Must happen AFTER other config steps

    def set_up_macro_display(self):
        self.ui.button_bar.set_installation_status(
            installed=self.addon.status() != Addon.Status.NOT_INSTALLED,
            available_branches=[],
            disabled=self.addon.is_disabled(),
            can_be_disabled=False,
        )
        self.ui.button_bar.set_can_run(True)

    def enable_clicked(self) -> None:
        """Called by the Enable button, enables this Addon and updates GUI to reflect
        that status."""
        self.addon.enable()
        self.ui.set_disabled(False)
        if self.original_disabled_state:
            self.ui.set_new_disabled_status(False)
            self.original_status = self.addon.status()
            self.addon.set_status(Addon.Status.PENDING_RESTART)
        else:
            self.addon.set_status(self.original_status)
        self.update_status.emit(self.addon)

    def disable_clicked(self) -> None:
        """Called by the Disable button, disables this Addon and updates the GUI to
        reflect that status."""
        self.addon.disable()
        self.ui.set_disabled(True)
        if not self.original_disabled_state:
            self.ui.set_new_disabled_status(True)
            self.original_status = self.addon.status()
            self.addon.set_status(Addon.Status.PENDING_RESTART)
        else:
            self.addon.set_status(self.original_status)
        self.update_status.emit(self.addon)

    def install_branch(self, branch: str):
        if self.addon.branch_display_name == branch:
            fci.Console.PrintMessage(
                f"Installing active branch {branch} for {self.addon.display_name}\n"
            )
            self.install.emit(self.addon)
            return
        if branch not in self.addon.sub_addons:
            fci.Console.PrintError(
                f"Internal error: branch {branch} not found in sub_addons list for addon {self.addon.display_name}.\n"
            )
            return
        fci.Console.PrintMessage(f"Installing sub-branch {branch} for {self.addon.display_name}\n")
        self.install.emit(self.addon.sub_addons[branch])

    def branch_changed(self, old_branch: str, name: str) -> None:
        """Displays a dialog confirming the branch changed, and tries to access the
        metadata file from that branch."""
        # See if this branch has a package.xml file:
        basedir = fci.getUserAppDataDir()
        path_to_metadata = os.path.join(basedir, "Mod", self.addon.name, "package.xml")
        if os.path.isfile(path_to_metadata):
            self.addon.load_metadata_file(path_to_metadata)
            self.addon.installed_version = self.addon.metadata.version
        else:
            self.addon.repo_type = Addon.Kind.WORKBENCH
            self.addon.metadata = None
            self.addon.installed_version = None
        self.addon.updated_timestamp = QtCore.QDateTime.currentDateTime().toSecsSinceEpoch()
        self.addon.branch = name
        self.addon.set_status(Addon.Status.PENDING_RESTART)
        self.ui.set_new_branch(name)
        self.update_status.emit(self.addon)
        QtWidgets.QMessageBox.information(
            self.ui,
            translate("AddonsInstaller", "Success"),
            translate(
                "AddonsInstaller",
                "Branch change succeeded.\n"
                "Moved\n"
                "from: {}\n"
                "to: {}\n"
                "Please restart to use the new version.",
            ).format(old_branch, name),
        )

    def display_repo_status(self, addon):
        self.update_status.emit(self.addon)
        self.show_addon(self.addon)
        self.ui.button_bar.set_update_check_status(False)



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2025 Jakub Michalski <jakub.j.michalski[at]gmail.com>   *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************

import FreeCAD
from FreeCAD import Rotation
from FreeCAD import Vector

import ObjectsFem

from . import manager
from .manager import get_meshname
from .manager import init_doc
from .meshes import generate_mesh


def get_information():
    return {
        "name": "Rotating Disc Cyclic Symmetry Centrif",
        "meshtype": "solid",
        "meshelement": "Tet10",
        "constraints": ["tie", "centrif", "displacement"],
        "solvers": ["ccxtools"],
        "material": "solid",
        "equations": ["mechanical"],
    }


def get_explanation(header=""):
    return (
        header
        + """

To run the example from Python console use:
from femexamples.ccx_disc_cyclic_symm_centrif import setup
setup()

Analytical solution - max von Mises stress 109.78 MPa = 1.0987e8 Pa

"""
    )


def setup(doc=None, solvertype="ccxtools"):

    # init FreeCAD document
    if doc is None:
        doc = init_doc()

    # explanation object
    # just keep the following line and change text string in get_explanation method
    manager.add_explanation_obj(doc, get_explanation(manager.get_header(get_information())))

    # geometric objects
    cylinder1 = doc.addObject("Part::Cylinder", "Cylinder1")
    cylinder1.Placement = FreeCAD.Placement(Vector(0, 0, 0), Rotation(Vector(0, 0, 1), 0))
    cylinder1.Radius = "400 mm"
    cylinder1.Height = "50 mm"
    cylinder1.Angle = "45 deg"
    

    cylinder2 = doc.addObject("Part::Cylinder", "Cylinder2")
    cylinder2.Placement = FreeCAD.Placement(Vector(0, 0, 0), Rotation(Vector(0, 0, 1), 0))
    cylinder2.Radius = "200 mm"
    cylinder2.Height = "50 mm"
    cylinder2.Angle = "45 deg"

    geom_obj = doc.addObject("Part::Cut", "Cut")
    geom_obj.Base = cylinder1
    geom_obj.Tool = cylinder2
    doc.recompute()

    if FreeCAD.GuiUp:
        geom_obj.ViewObject.Document.activeView().viewAxonometric()
        geom_obj.ViewObject.Document.activeView().fitAll()

    # line for centrif axis
    axis = doc.addObject("Part::Line", "Axis")
    axis.Placement = FreeCAD.Placement(Vector(0, 0, 0), Rotation(Vector(0, 0, 1), 0))
    axis.X1 = "0.00 mm"
    axis.Y1 = "0.00 mm"
    axis.Z1 = "0.00 mm"
    axis.X2 = "0.00 mm"
    axis.Y2 = "0.00 mm"
    axis.Z2 = "50.00 mm"
    doc.recompute()
    if FreeCAD.GuiUp:
        axis.ViewObject.LineWidth = 5.0
        axis.ViewObject.LineColor = (1.0, 0.0, 0.0)

    # analysis
    analysis = ObjectsFem.makeAnalysis(doc, "Analysis")
    if FreeCAD.GuiUp:
        import FemGui

        FemGui.setActiveAnalysis(analysis)

    # solver
    if solvertype == "ccxtools":
        solver_obj = ObjectsFem.makeSolverCalculiXCcxTools(doc, "CalculiXCcxTools")
        solver_obj.AnalysisType = "static"
        analysis.addObject(solver_obj)
    else:
        FreeCAD.Console.PrintWarning(
            "Unknown or unsupported solver type: {}. "
            "No solver object was created.\n".format(solvertype)
        )

    # material
    material_obj = ObjectsFem.makeMaterialSolid(doc, "Steel")
    mat = material_obj.Material
    mat["Name"] = "Calculix-Steel"
    mat["YoungsModulus"] = "210000 MPa"
    mat["PoissonRatio"] = "0.30"
    mat["Density"] = "8000 kg/m^3"
    material_obj.Material = mat
    material_obj.References = [(geom_obj, "Solid1")]
    analysis.addObject(material_obj)

    # constraint tie
    con_tie = ObjectsFem.makeConstraintTie(doc, "ConstraintTie")
    con_tie.References = [
        (geom_obj, "Face2"),
        (geom_obj, "Face3"),
    ]
    con_tie.Tolerance = 0.2
    con_tie.CyclicSymmetry = True
    con_tie.Sectors = 8
    con_tie.ConnectedSectors = 1
    analysis.addObject(con_tie)
    
    # constraint centrif
    con_centrif = ObjectsFem.makeConstraintCentrif(doc, "ConstraintCentrif")
    con_centrif.References = [(geom_obj, "Solid1")]
    con_centrif.RotationFrequency = "50 Hz"
    con_centrif.RotationAxis = [(axis, "Edge1")]
    analysis.addObject(con_centrif)
    
    # constraint displacement
    con_disp = ObjectsFem.makeConstraintDisplacement(doc, "ConstraintDisplacement")
    con_disp.References = [(geom_obj, ("Vertex8"))]
    analysis.addObject(con_disp)
    con_disp.zFree = False
    con_disp.zDisplacement = "0.00 mm"

    # mesh
    femmesh_obj = analysis.addObject(ObjectsFem.makeMeshGmsh(doc, get_meshname()))[0]
    femmesh_obj.Shape = geom_obj
    femmesh_obj.ElementOrder = "2nd"
    femmesh_obj.CharacteristicLengthMax = "10 mm"
    femmesh_obj.ViewObject.Visibility = False

    # generate the mesh
    generate_mesh.mesh_from_mesher(femmesh_obj, "gmsh")

    doc.recompute()
    return doc




# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2017 sliptonic <shopinthewoods@gmail.com>               *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

import FreeCAD
import Path
import Path.Dressup.Gui.Dogbone as PathDressupDogbone
import Path.Main.Job as PathJob
import Path.Op.Profile as PathProfile

from CAMTests.PathTestUtils import PathTestBase


class TestProfile:
    def __init__(self, side, direction, path):
        self.Side = side
        self.Direction = direction
        self.Path = Path.Path(path)
        self.ToolController = None  # default tool 5mm
        self.Name = "Profile"


class TestFeature:
    def __init__(self):
        self.Path = Path.Path()

    def addProperty(self, typ, name, category, tip):
        setattr(self, name, None)

    def setEditorMode(self, prop, mode):
        pass


class TestDressupDogbone(PathTestBase):
    """Unit tests for the Dogbone dressup."""

    def formatBone(self, bone):
        return "%d: (%.2f, %.2f)" % (bone[0], bone[1][0], bone[1][1])

    def test00(self):
        """Verify bones are inserted for simple moves."""
        base = TestProfile(
            "Inside",
            "CW",
            """
        G0 X10 Y10 Z10
        G1 Z0
        G1 Y100
        G1 X12
        G1 Y10
        G1 X10
        G1 Z10
        """,
        )
        obj = TestFeature()
        db = PathDressupDogbone.ObjectDressup(obj, base)
        db.setup(obj, True)
        db.execute(obj, False)
        self.assertEqual(len(db.bones), 4)
        self.assertEqual("1: (10.00, 100.00)", self.formatBone(db.bones[0]))
        self.assertEqual("2: (12.00, 100.00)", self.formatBone(db.bones[1]))
        self.assertEqual("3: (12.00, 10.00)", self.formatBone(db.bones[2]))
        self.assertEqual("4: (10.00, 10.00)", self.formatBone(db.bones[3]))

    def test01(self):
        """Verify bones are inserted if hole ends with rapid move out."""
        base = TestProfile(
            "Inside",
            "CW",
            """
        G0 X10 Y10 Z10
        G1 Z0
        G1 Y100
        G1 X12
        G1 Y10
        G1 X10
        G0 Z10
        """,
        )
        obj = TestFeature()
        db = PathDressupDogbone.ObjectDressup(obj, base)
        db.setup(obj, True)
        db.execute(obj, False)
        self.assertEqual(len(db.bones), 4)
        self.assertEqual("1: (10.00, 100.00)", self.formatBone(db.bones[0]))
        self.assertEqual("2: (12.00, 100.00)", self.formatBone(db.bones[1]))
        self.assertEqual("3: (12.00, 10.00)", self.formatBone(db.bones[2]))
        self.assertEqual("4: (10.00, 10.00)", self.formatBone(db.bones[3]))

    def test02(self):
        """Verify bones are correctly generated for a Profile."""
        doc = FreeCAD.newDocument("TestDressupDogbone")

        # This is a real world test to make sure none of the tool chain broke
        box0 = doc.addObject("Part::Box", "Box")
        box0.Width = 100
        box0.Length = 100
        box0.Height = 10
        box1 = doc.addObject("Part::Box", "Box")
        box1.Width = 50
        box1.Length = 50
        box1.Height = 20
        box1.Placement = FreeCAD.Placement(
            FreeCAD.Vector(25, 25, -5), FreeCAD.Rotation(FreeCAD.Vector(0, 0, 1), 0)
        )
        doc.recompute()
        cut = doc.addObject("Part::Cut", "Cut")
        cut.Base = box0
        cut.Tool = box1
        doc.recompute()

        for i in range(11):
            face = "Face%d" % (i + 1)
            f = cut.Shape.getElement(face)
            if f.Surface.Axis == FreeCAD.Vector(0, 0, 1) and f.Orientation == "Forward":
                break

        PathJob.Create("Job", [cut], None)

        profile = PathProfile.Create("Profile")
        profile.Base = (cut, face)
        profile.StepDown = 5
        # set start and final depth in order to eliminate effects of stock (and its default values)
        profile.setExpression("StartDepth", None)
        profile.StartDepth = 10
        profile.setExpression("FinalDepth", None)
        profile.FinalDepth = 0

        profile.processHoles = True
        profile.processPerimeter = True
        doc.recompute()

        dogbone = PathDressupDogbone.Create(profile)
        doc.recompute()

        dog = dogbone.Proxy
        locs = sorted([bone[1] for bone in dog.bones], key=lambda xy: xy[0] * 1000 + xy[1])

        def formatBoneLoc(pt):
            return "(%.2f, %.2f)" % (pt[0], pt[1])

        # Make sure we get 8 bones, 2 in each corner (different heights)
        # with start point changes it passes back over the same spot multiple times, so just make sure they are in the right locations
        # self.assertEqual(len(locs), 8)
        self.assertEqual("(27.50, 27.50)", formatBoneLoc(locs[0]))
        self.assertEqual("(27.50, 27.50)", formatBoneLoc(locs[1]))
        self.assertEqual("(27.50, 72.50)", formatBoneLoc(locs[2]))
        self.assertEqual("(27.50, 72.50)", formatBoneLoc(locs[3]))
        self.assertEqual("(72.50, 27.50)", formatBoneLoc(locs[4]))
        self.assertEqual("(72.50, 27.50)", formatBoneLoc(locs[5]))
        self.assertEqual("(72.50, 72.50)", formatBoneLoc(locs[6]))
        self.assertEqual("(72.50, 72.50)", formatBoneLoc(locs[7]))

        FreeCAD.closeDocument("TestDressupDogbone")

    def test03(self):
        """Verify no bone is inserted for straight move interrupted by plunge."""
        base = TestProfile(
            "Inside",
            "CW",
            """
        G0 X10 Y10 Z10
        G1 Z0
        G1 X0   ( start)
        G1 Y0
        G1 X15
        G1 Y10
        G1 X10  ( straight line move to start)
        G0 Z10
        """,
        )
        obj = TestFeature()
        db = PathDressupDogbone.ObjectDressup(obj, base)
        db.setup(obj, True)
        db.execute(obj, False)
        self.assertEqual(len(db.bones), 0)

    def test04(self):
        """Verify can handle comments between moves"""
        base = TestProfile(
            "Inside",
            "CW",
            """
        G0 X10 Y10 Z10
        G1 Z0
        G1 X20
        G1 Y0
        G1 X10
        G1 Y10
        G1 Z10
        """,
        )
        obj = TestFeature()
        db = PathDressupDogbone.ObjectDressup(obj, base)
        db.setup(obj, True)
        db.execute(obj, False)
        self.assertEqual(len(db.bones), 4)
        self.assertEqual("1: (20.00, 10.00)", self.formatBone(db.bones[0]))
        self.assertEqual("2: (20.00, 0.00)", self.formatBone(db.bones[1]))
        self.assertEqual("3: (10.00, 0.00)", self.formatBone(db.bones[2]))
        self.assertEqual("4: (10.00, 10.00)", self.formatBone(db.bones[3]))

        base = TestProfile(
            "Inside",
            "CW",
            """
        G0 X10 Y10 Z10
        G1 Z0
        G1 X20
        G1 Y0
        G1 X10
        (some comment or other should not change the output)
        G1 Y10
        G1 Z10
        """,
        )
        obj = TestFeature()
        db = PathDressupDogbone.ObjectDressup(obj, base)
        db.setup(obj, True)
        db.execute(obj, False)
        self.assertEqual(len(db.bones), 4)
        self.assertEqual("1: (20.00, 10.00)", self.formatBone(db.bones[0]))
        self.assertEqual("2: (20.00, 0.00)", self.formatBone(db.bones[1]))
        self.assertEqual("3: (10.00, 0.00)", self.formatBone(db.bones[2]))
        self.assertEqual("4: (10.00, 10.00)", self.formatBone(db.bones[3]))

    def test05(self):
        """Verify can handle noops between moves"""
        base = TestProfile(
            "Inside",
            "CW",
            """
        G0 X10 Y10 Z10
        G1 Z0
        G1 X20
        G1 Y0
        G1 X10
        G1 Y10
        G1 Z10
        """,
        )
        obj = TestFeature()
        db = PathDressupDogbone.ObjectDressup(obj, base)
        db.setup(obj, True)
        db.execute(obj, False)
        self.assertEqual(len(db.bones), 4)
        self.assertEqual("1: (20.00, 10.00)", self.formatBone(db.bones[0]))
        self.assertEqual("2: (20.00, 0.00)", self.formatBone(db.bones[1]))
        self.assertEqual("3: (10.00, 0.00)", self.formatBone(db.bones[2]))
        self.assertEqual("4: (10.00, 10.00)", self.formatBone(db.bones[3]))

        base = TestProfile(
            "Inside",
            "CW",
            """
        G0 X10 Y10 Z10
        G1 Z0
        G1 X20
        G1 Y0
        G1 X10
        G1 X10
        G1 Y10
        G1 Z10
        """,
        )
        obj = TestFeature()
        db = PathDressupDogbone.ObjectDressup(obj, base)
        db.setup(obj, True)
        db.execute(obj, False)
        self.assertEqual(len(db.bones), 4)
        self.assertEqual("1: (20.00, 10.00)", self.formatBone(db.bones[0]))
        self.assertEqual("2: (20.00, 0.00)", self.formatBone(db.bones[1]))
        self.assertEqual("3: (10.00, 0.00)", self.formatBone(db.bones[2]))
        self.assertEqual("4: (10.00, 10.00)", self.formatBone(db.bones[3]))



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *                                                                         *
# *   Copyright (c) 2011 Yorik van Havre <yorik@uncreated.net>              *
# *                                                                         *
# *   This file is part of FreeCAD.                                         *
# *                                                                         *
# *   FreeCAD is free software: you can redistribute it and/or modify it    *
# *   under the terms of the GNU Lesser General Public License as           *
# *   published by the Free Software Foundation, either version 2.1 of the  *
# *   License, or (at your option) any later version.                       *
# *                                                                         *
# *   FreeCAD is distributed in the hope that it will be useful, but        *
# *   WITHOUT ANY WARRANTY; without even the implied warranty of            *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU      *
# *   Lesser General Public License for more details.                       *
# *                                                                         *
# *   You should have received a copy of the GNU Lesser General Public      *
# *   License along with FreeCAD. If not, see                               *
# *   <https://www.gnu.org/licenses/>.                                      *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD Axis System"
__author__ = "Yorik van Havre"
__url__ = "https://www.freecad.org"

## @package ArchAxisSystem
#  \ingroup ARCH
#  \brief Axis system for the Arch workbench
#
#  This module provides tools to build axis systems
#  An axis system is a collection of multiple axes

import FreeCAD
import DraftGeomUtils

if FreeCAD.GuiUp:
    from pivy import coin
    from PySide import QtCore, QtGui
    from PySide.QtCore import QT_TRANSLATE_NOOP
    import FreeCADGui
    import Draft
    from draftutils.translate import translate
else:
    # \cond
    def translate(ctxt, txt):
        return txt

    def QT_TRANSLATE_NOOP(ctxt, txt):
        return txt

    # \endcond


class _AxisSystem:
    "The Axis System object"

    def __init__(self, obj):

        obj.Proxy = self
        self.Type = "AxisSystem"
        self.setProperties(obj)

    def setProperties(self, obj):

        pl = obj.PropertiesList
        if not "Axes" in pl:
            obj.addProperty(
                "App::PropertyLinkList",
                "Axes",
                "AxisSystem",
                QT_TRANSLATE_NOOP("App::Property", "The axes this system is made of"),
                locked=True,
            )
        if not "Placement" in pl:
            obj.addProperty(
                "App::PropertyPlacement",
                "Placement",
                "AxisSystem",
                QT_TRANSLATE_NOOP("App::Property", "The placement of this axis system"),
                locked=True,
            )

    def onDocumentRestored(self, obj):

        self.setProperties(obj)

    def execute(self, obj):

        pass

    def onBeforeChange(self, obj, prop):

        if prop == "Placement":
            self.Placement = obj.Placement

    def onChanged(self, obj, prop):

        if prop == "Placement":
            if hasattr(self, "Placement"):
                delta = obj.Placement.multiply(self.Placement.inverse())
                for o in obj.Axes:
                    o.Placement = delta.multiply(o.Placement)

    def dumps(self):

        return None

    def loads(self, state):

        self.Type = "AxisSystem"

    def getPoints(self, obj):
        "returns the gridpoints of linked axes"

        pts = []
        if len(obj.Axes) == 1:
            for e in obj.Axes[0].Shape.Edges:
                pts.append(e.Vertexes[0].Point)
        elif len(obj.Axes) == 2:
            set1 = obj.Axes[0].Shape.Edges  # X
            set2 = obj.Axes[1].Shape.Edges  # Y
            for e1 in set1:
                for e2 in set2:
                    pts.extend(DraftGeomUtils.findIntersection(e1, e2))
        elif len(obj.Axes) == 3:
            set1 = obj.Axes[0].Shape.Edges  # X
            set2 = obj.Axes[1].Shape.Edges  # Y
            set3 = obj.Axes[2].Shape.Edges  # Z
            bset = []
            cv = None
            for e1 in set1:
                for e2 in set2:
                    bset.extend(DraftGeomUtils.findIntersection(e1, e2))
            for e3 in set3:
                if not cv:
                    cv = e3.Vertexes[0].Point
                    pts.extend(bset)
                else:
                    v = e3.Vertexes[0].Point.sub(cv)
                    pts.extend([p.add(v) for p in bset])
        return pts

    def getAxisData(self, obj):
        data = []
        for axis in obj.Axes:
            if hasattr(axis, "Proxy") and hasattr(axis.Proxy, "getAxisData"):
                data.append(axis.Proxy.getAxisData(axis))
        return data


class _ViewProviderAxisSystem:
    "A View Provider for the Axis object"

    def __init__(self, vobj):

        vobj.Proxy = self

    def getIcon(self):

        import Arch_rc

        return ":/icons/Arch_Axis_System_Tree.svg"

    def claimChildren(self):

        if hasattr(self, "axes"):
            return self.axes
        return []

    def attach(self, vobj):
        self.Object = vobj.Object
        self.axes = vobj.Object.Axes
        vobj.addDisplayMode(coin.SoSeparator(), "Default")

    def getDisplayModes(self, vobj):

        return ["Default"]

    def getDefaultDisplayMode(self):

        return "Default"

    def setDisplayMode(self, mode):

        return mode

    def updateData(self, obj, prop):

        self.axes = obj.Axes

    def onChanged(self, vobj, prop):

        if prop == "Visibility":
            for o in vobj.Object.Axes:
                o.ViewObject.Visibility = vobj.Visibility

    def setEdit(self, vobj, mode):
        if mode != 0:
            return None

        taskd = AxisSystemTaskPanel(vobj.Object)
        FreeCADGui.Control.showDialog(taskd)
        return True

    def unsetEdit(self, vobj, mode):
        if mode != 0:
            return None

        FreeCADGui.Control.closeDialog()
        return True

    def doubleClicked(self, vobj):
        self.edit()
        return True

    def setupContextMenu(self, vobj, menu):
        if FreeCADGui.activeWorkbench().name() != "BIMWorkbench":
            return
        actionEdit = QtGui.QAction(translate("Arch", "Edit"), menu)
        QtCore.QObject.connect(actionEdit, QtCore.SIGNAL("triggered()"), self.edit)
        menu.addAction(actionEdit)

    def edit(self):
        FreeCADGui.ActiveDocument.setEdit(self.Object, 0)

    def dumps(self):

        return None

    def loads(self, state):

        return None


class AxisSystemTaskPanel:
    """A TaskPanel for all the section plane object"""

    def __init__(self, obj):

        self.obj = obj
        self.form = QtGui.QWidget()
        self.form.setObjectName("Axis System")
        self.grid = QtGui.QGridLayout(self.form)
        self.grid.setObjectName("grid")
        self.title = QtGui.QLabel(self.form)
        self.grid.addWidget(self.title, 0, 0, 1, 2)

        # tree
        self.tree = QtGui.QTreeWidget(self.form)
        self.grid.addWidget(self.tree, 1, 0, 1, 2)
        self.tree.setColumnCount(1)
        self.tree.header().hide()

        # buttons
        self.addButton = QtGui.QPushButton(self.form)
        self.addButton.setObjectName("addButton")
        self.addButton.setIcon(QtGui.QIcon(":/icons/Arch_Add.svg"))
        self.grid.addWidget(self.addButton, 3, 0, 1, 1)

        self.delButton = QtGui.QPushButton(self.form)
        self.delButton.setObjectName("delButton")
        self.delButton.setIcon(QtGui.QIcon(":/icons/Arch_Remove.svg"))
        self.grid.addWidget(self.delButton, 3, 1, 1, 1)

        QtCore.QObject.connect(self.addButton, QtCore.SIGNAL("clicked()"), self.addElement)
        QtCore.QObject.connect(self.delButton, QtCore.SIGNAL("clicked()"), self.removeElement)
        self.update()

    def isAllowedAlterSelection(self):

        return True

    def isAllowedAlterView(self):

        return True

    def getStandardButtons(self):

        return QtGui.QDialogButtonBox.Ok

    def getIcon(self, obj):

        if hasattr(obj.ViewObject, "Proxy"):
            return QtGui.QIcon(obj.ViewObject.Proxy.getIcon())
        elif obj.isDerivedFrom("Sketcher::SketchObject"):
            return QtGui.QIcon(":/icons/Sketcher_Sketch.svg")
        elif obj.isDerivedFrom("App::DocumentObjectGroup"):
            return QtGui.QApplication.style().standardIcon(QtGui.QStyle.SP_DirIcon)
        elif hasattr(obj.ViewObject, "Icon"):
            return QtGui.QIcon(obj.ViewObject.Icon)
        return QtGui.QIcon(":/icons/Part_3D_object.svg")

    def update(self):

        self.tree.clear()
        if self.obj:
            for o in self.obj.Axes:
                item = QtGui.QTreeWidgetItem(self.tree)
                item.setText(0, o.Label)
                item.setToolTip(0, o.Name)
                item.setIcon(0, self.getIcon(o))
        self.retranslateUi(self.form)

    def addElement(self):

        if self.obj:
            for o in FreeCADGui.Selection.getSelection():
                if not (o in self.obj.Axes) and (o != self.obj):
                    g = self.obj.Axes
                    g.append(o)
                    self.obj.Axes = g
            self.update()

    def removeElement(self):

        if self.obj:
            it = self.tree.currentItem()
            if it:
                o = FreeCAD.ActiveDocument.getObject(str(it.toolTip(0)))
                if o in self.obj.Axes:
                    g = self.obj.Axes
                    g.remove(o)
                    self.obj.Axes = g
            self.update()

    def accept(self):

        FreeCAD.ActiveDocument.recompute()
        FreeCADGui.ActiveDocument.resetEdit()
        return True

    def retranslateUi(self, TaskPanel):

        TaskPanel.setWindowTitle(QtGui.QApplication.translate("Arch", "Axes", None))
        self.delButton.setText(QtGui.QApplication.translate("Arch", "Remove", None))
        self.addButton.setText(QtGui.QApplication.translate("Arch", "Add", None))
        self.title.setText(QtGui.QApplication.translate("Arch", "Axis system components", None))



# ***************************************************************************
# *   Copyright (c) 2023 Uwe Stöhr <uwestoehr@lyx.org>                      *
# *   Copyright (c) 2025 Mario Passaglia <mpassaglia[at]cbc.uba.ar>         *
# *                                                                         *
# *   This file is part of the FreeCAD CAx development system.              *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

__title__ = "FreeCAD FEM constraint magnetization task panel for the document object"
__author__ = "Uwe Stöhr, Mario Passaglia"
__url__ = "https://www.freecad.org"

## @package task_constraint_magnetization
#  \ingroup FEM
#  \brief task panel for constraint magnetization object

from PySide import QtCore

import FreeCAD
import FreeCADGui

from femguiutils import selection_widgets

from femtools import membertools
from . import base_femtaskpanel


class _TaskPanel(base_femtaskpanel._BaseTaskPanel):

    def __init__(self, obj):
        super().__init__(obj)

        self.parameter_widget = FreeCADGui.PySideUic.loadUi(
            FreeCAD.getHomePath() + "Mod/Fem/Resources/ui/Magnetization.ui"
        )

        # geometry selection widget
        # magnetization is always a body force for 3D, therefore only allow solid
        self._selection_widget = selection_widgets.GeometryElementsSelection(
            obj.References, ["Solid", "Face"], True, False
        )

        # form made from param and selection widget
        self.form = [self.parameter_widget, self._selection_widget]

        analysis = obj.getParentGroup()
        self._mesh = None
        self._part = None
        if analysis is not None:
            self._mesh = membertools.get_single_member(analysis, "Fem::FemMeshObject")
        if self._mesh is not None:
            self._part = self._mesh.Shape
        self._partVisible = None
        self._meshVisible = None

        QtCore.QObject.connect(
            self.parameter_widget.ckb_magnetization_1,
            QtCore.SIGNAL("toggled(bool)"),
            self.magnetization_1_enabled_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.qsb_magnetization_re_1,
            QtCore.SIGNAL("valueChanged(Base::Quantity)"),
            self.magnetization_re_1_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.ckb_magnetization_2,
            QtCore.SIGNAL("toggled(bool)"),
            self.magnetization_2_enabled_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.qsb_magnetization_re_2,
            QtCore.SIGNAL("valueChanged(Base::Quantity)"),
            self.magnetization_re_2_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.ckb_magnetization_3,
            QtCore.SIGNAL("toggled(bool)"),
            self.magnetization_3_enabled_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.qsb_magnetization_re_3,
            QtCore.SIGNAL("valueChanged(Base::Quantity)"),
            self.magnetization_re_3_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.qsb_magnetization_im_1,
            QtCore.SIGNAL("valueChanged(Base::Quantity)"),
            self.magnetization_im_1_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.qsb_magnetization_im_2,
            QtCore.SIGNAL("valueChanged(Base::Quantity)"),
            self.magnetization_im_2_changed,
        )
        QtCore.QObject.connect(
            self.parameter_widget.qsb_magnetization_im_3,
            QtCore.SIGNAL("valueChanged(Base::Quantity)"),
            self.magnetization_im_3_changed,
        )

        self.init_parameter_widget()

    def open(self):
        if self._mesh is not None and self._part is not None:
            self._meshVisible = self._mesh.ViewObject.isVisible()
            self._partVisible = self._part.ViewObject.isVisible()
            self._mesh.ViewObject.hide()
            self._part.ViewObject.show()

    def reject(self):
        self._restoreVisibility()
        self._selection_widget.finish_selection()
        return super().reject()

    def accept(self):
        if self.obj.References != self._selection_widget.references:
            self.obj.References = self._selection_widget.references
        self._set_params()
        self._selection_widget.finish_selection()
        self._restoreVisibility()
        return super().accept()

    def _restoreVisibility(self):
        if self._mesh is not None and self._part is not None:
            if self._meshVisible:
                self._mesh.ViewObject.show()
            else:
                self._mesh.ViewObject.hide()
            if self._partVisible:
                self._part.ViewObject.show()
            else:
                self._part.ViewObject.hide()

    def _get_params(self):
        self.magnetization_re_1 = self.obj.Magnetization_re_1
        self.magnetization_re_2 = self.obj.Magnetization_re_2
        self.magnetization_re_3 = self.obj.Magnetization_re_3
        self.magnetization_im_1 = self.obj.Magnetization_im_1
        self.magnetization_im_2 = self.obj.Magnetization_im_2
        self.magnetization_im_3 = self.obj.Magnetization_im_3

        self.magnetization_1_enabled = self.obj.EnableMagnetization_1
        self.magnetization_2_enabled = self.obj.EnableMagnetization_2
        self.magnetization_3_enabled = self.obj.EnableMagnetization_3

    def _set_params(self):
        self.obj.Magnetization_re_1 = self.magnetization_re_1
        self.obj.Magnetization_re_2 = self.magnetization_re_2
        self.obj.Magnetization_re_3 = self.magnetization_re_3
        self.obj.Magnetization_im_1 = self.magnetization_im_1
        self.obj.Magnetization_im_2 = self.magnetization_im_2
        self.obj.Magnetization_im_3 = self.magnetization_im_3

        self.obj.EnableMagnetization_1 = self.magnetization_1_enabled
        self.obj.EnableMagnetization_2 = self.magnetization_2_enabled
        self.obj.EnableMagnetization_3 = self.magnetization_3_enabled

    def init_parameter_widget(self):
        self._get_params()

        self.parameter_widget.qsb_magnetization_re_1.setProperty("value", self.magnetization_re_1)
        self.parameter_widget.qsb_magnetization_re_1.setEnabled(self.magnetization_1_enabled)
        FreeCADGui.ExpressionBinding(self.parameter_widget.qsb_magnetization_re_1).bind(
            self.obj, "Magnetization_re_1"
        )

        self.parameter_widget.qsb_magnetization_re_2.setProperty("value", self.magnetization_re_2)
        self.parameter_widget.qsb_magnetization_re_2.setEnabled(self.magnetization_2_enabled)
        FreeCADGui.ExpressionBinding(self.parameter_widget.qsb_magnetization_re_2).bind(
            self.obj, "Magnetization_re_2"
        )

        self.parameter_widget.qsb_magnetization_re_3.setProperty("value", self.magnetization_re_3)
        self.parameter_widget.qsb_magnetization_re_3.setEnabled(self.magnetization_3_enabled)
        FreeCADGui.ExpressionBinding(self.parameter_widget.qsb_magnetization_re_3).bind(
            self.obj, "Magnetization_re_3"
        )

        self.parameter_widget.qsb_magnetization_im_1.setProperty("value", self.magnetization_im_1)
        self.parameter_widget.qsb_magnetization_im_1.setEnabled(self.magnetization_1_enabled)
        FreeCADGui.ExpressionBinding(self.parameter_widget.qsb_magnetization_im_1).bind(
            self.obj, "Magnetization_im_1"
        )

        self.parameter_widget.qsb_magnetization_im_2.setProperty("value", self.magnetization_im_2)
        self.parameter_widget.qsb_magnetization_im_2.setEnabled(self.magnetization_2_enabled)
        FreeCADGui.ExpressionBinding(self.parameter_widget.qsb_magnetization_im_2).bind(
            self.obj, "Magnetization_im_2"
        )

        self.parameter_widget.qsb_magnetization_im_3.setProperty("value", self.magnetization_im_3)
        self.parameter_widget.qsb_magnetization_im_3.setEnabled(self.magnetization_3_enabled)
        FreeCADGui.ExpressionBinding(self.parameter_widget.qsb_magnetization_im_3).bind(
            self.obj, "Magnetization_im_3"
        )

        self.parameter_widget.ckb_magnetization_1.setChecked(self.magnetization_1_enabled)
        self.parameter_widget.ckb_magnetization_2.setChecked(self.magnetization_2_enabled)
        self.parameter_widget.ckb_magnetization_3.setChecked(self.magnetization_3_enabled)

    def magnetization_1_enabled_changed(self, value):
        self.magnetization_1_enabled = value
        self.parameter_widget.qsb_magnetization_re_1.setEnabled(value)
        self.parameter_widget.qsb_magnetization_im_1.setEnabled(value)

    def magnetization_2_enabled_changed(self, value):
        self.magnetization_2_enabled = value
        self.parameter_widget.qsb_magnetization_re_2.setEnabled(value)
        self.parameter_widget.qsb_magnetization_im_2.setEnabled(value)

    def magnetization_3_enabled_changed(self, value):
        self.magnetization_3_enabled = value
        self.parameter_widget.qsb_magnetization_re_3.setEnabled(value)
        self.parameter_widget.qsb_magnetization_im_3.setEnabled(value)

    def magnetization_re_1_changed(self, value):
        self.magnetization_re_1 = value

    def magnetization_re_2_changed(self, value):
        self.magnetization_re_2 = value

    def magnetization_re_3_changed(self, value):
        self.magnetization_re_3 = value

    def magnetization_im_1_changed(self, value):
        self.magnetization_im_1 = value

    def magnetization_im_2_changed(self, value):
        self.magnetization_im_2 = value

    def magnetization_im_3_changed(self, value):
        self.magnetization_im_3 = value



# SPDX-License-Identifier: LGPL-2.1-or-later

# ***************************************************************************
# *   Copyright (c) 2022 sliptonic <shopinthewoods@gmail.com>               *
# *   Copyright (c) 2022 Larry Woestman <LarryWoestman2@gmail.com>          *
# *                                                                         *
# *   This program is free software; you can redistribute it and/or modify  *
# *   it under the terms of the GNU Lesser General Public License (LGPL)    *
# *   as published by the Free Software Foundation; either version 2 of     *
# *   the License, or (at your option) any later version.                   *
# *   for detail see the LICENCE text file.                                 *
# *                                                                         *
# *   This program is distributed in the hope that it will be useful,       *
# *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
# *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
# *   GNU Library General Public License for more details.                  *
# *                                                                         *
# *   You should have received a copy of the GNU Library General Public     *
# *   License along with this program; if not, write to the Free Software   *
# *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  *
# *   USA                                                                   *
# *                                                                         *
# ***************************************************************************

from importlib import reload

import FreeCAD

import Path
import CAMTests.PathTestUtils as PathTestUtils
from Path.Post.scripts import mach3_mach4_post as postprocessor


Path.Log.setLevel(Path.Log.Level.DEBUG, Path.Log.thisModule())
Path.Log.trackModule(Path.Log.thisModule())


class TestMach3Mach4Post(PathTestUtils.PathTestBase):
    @classmethod
    def setUpClass(cls):
        """setUpClass()...
        This method is called upon instantiation of this test class.  Add code
        and objects here that are needed for the duration of the test() methods
        in this class.  In other words, set up the 'global' test environment
        here; use the `setUp()` method to set up a 'local' test environment.
        This method does not have access to the class `self` reference, but it
        is able to call static methods within this same class.
        """

        # Open existing FreeCAD document with test geometry
        FreeCAD.newDocument("Unnamed")

    @classmethod
    def tearDownClass(cls):
        """tearDownClass()...
        This method is called prior to destruction of this test class.  Add
        code and objects here that cleanup the test environment after the
        test() methods in this class have been executed.  This method does
        not have access to the class `self` reference.  This method is able
        to call static methods within this same class.
        """
        # Close geometry document without saving
        FreeCAD.closeDocument(FreeCAD.ActiveDocument.Name)

    # Setup and tear down methods called before and after each unit test
    def setUp(self):
        """setUp()...
        This method is called prior to each `test()` method.  Add code and
        objects here that are needed for multiple `test()` methods.
        """
        self.doc = FreeCAD.ActiveDocument
        self.con = FreeCAD.Console
        self.docobj = FreeCAD.ActiveDocument.addObject("Path::Feature", "testpath")
        reload(
            postprocessor
        )  # technical debt.  This shouldn't be necessary but here to bypass a bug

    def tearDown(self):
        """tearDown()...
        This method is called after each test() method. Add cleanup instructions here.
        Such cleanup instructions will likely undo those in the setUp() method.
        """
        FreeCAD.ActiveDocument.removeObject("testpath")

    def test000(self):
        """Test Output Generation.
        Empty path.  Produces only the preamble and postable.
        """

        self.docobj.Path = Path.Path([])
        postables = [self.docobj]

        # Test generating with header
        # Header contains a time stamp that messes up unit testing.
        # Only test length of result.
        args = "--no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        self.assertTrue(len(gcode.splitlines()) == 13)

        # Test without header
        expected = """(begin preamble)
G17 G54 G40 G49 G80 G90
G21
(begin operation: testpath)
(machine: mach3_4, mm/min)
(finish operation: testpath)
(begin postamble)
M05
G17 G54 G90 G80 G40
M2
"""

        self.docobj.Path = Path.Path([])
        postables = [self.docobj]

        args = "--no-header --no-show-editor"
        # args = ("--no-header --no-comments --no-show-editor --precision=2")
        gcode = postprocessor.export(postables, "-", args)
        self.assertEqual(gcode, expected)

        # test without comments
        expected = """G17 G54 G40 G49 G80 G90
G21
M05
G17 G54 G90 G80 G40
M2
"""

        args = "--no-header --no-comments --no-show-editor"
        # args = ("--no-header --no-comments --no-show-editor --precision=2")
        gcode = postprocessor.export(postables, "-", args)
        self.assertEqual(gcode, expected)

    def test010(self):
        """Test command Generation.
        Test Precision
        """
        c = Path.Command("G0 X10 Y20 Z30")

        self.docobj.Path = Path.Path([c])
        postables = [self.docobj]

        args = "--no-header --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[5]
        expected = "G0 X10.000 Y20.000 Z30.000"
        self.assertEqual(result, expected)

        args = "--no-header --precision=2 --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[5]
        expected = "G0 X10.00 Y20.00 Z30.00"
        self.assertEqual(result, expected)

    def test020(self):
        """
        Test Line Numbers
        """
        c = Path.Command("G0 X10 Y20 Z30")

        self.docobj.Path = Path.Path([c])
        postables = [self.docobj]

        args = "--no-header --line-numbers --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[5]
        expected = "N160  G0 X10.000 Y20.000 Z30.000"
        self.assertEqual(result, expected)

    def test030(self):
        """
        Test Pre-amble
        """

        self.docobj.Path = Path.Path([])
        postables = [self.docobj]

        args = "--no-header --no-comments --preamble='G18 G55' --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[0]
        self.assertEqual(result, "G18 G55")

    def test040(self):
        """
        Test Post-amble
        """
        self.docobj.Path = Path.Path([])
        postables = [self.docobj]
        args = "--no-header --no-comments --postamble='G0 Z50\nM2' --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[-2]
        self.assertEqual(result, "G0 Z50")
        self.assertEqual(gcode.splitlines()[-1], "M2")

    def test050(self):
        """
        Test inches
        """

        c = Path.Command("G0 X10 Y20 Z30")
        self.docobj.Path = Path.Path([c])
        postables = [self.docobj]

        args = "--no-header --inches --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        self.assertEqual(gcode.splitlines()[2], "G20")

        result = gcode.splitlines()[5]
        expected = "G0 X0.3937 Y0.7874 Z1.1811"
        self.assertEqual(result, expected)

        # Technical debt.   The following test fails.  Precision not working
        # with imperial units.

        # args = ("--no-header --inches --precision=2 --no-show-editor")
        # gcode = postprocessor.export(postables, "-", args)
        # result = gcode.splitlines()[5]
        # expected = "G0 X0.39 Y0.79 Z1.18"
        # self.assertEqual(result, expected)

    def test060(self):
        """
        Test test modal
        Suppress the command name if the same as previous
        """
        c = Path.Command("G0 X10 Y20 Z30")
        c1 = Path.Command("G0 X10 Y30 Z30")

        self.docobj.Path = Path.Path([c, c1])
        postables = [self.docobj]

        args = "--no-header --modal --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[6]
        expected = "X10.000 Y30.000 Z30.000"
        self.assertEqual(result, expected)

    def test070(self):
        """
        Test axis modal
        Suppress the axis coordinate if the same as previous
        """
        c = Path.Command("G0 X10 Y20 Z30")
        c1 = Path.Command("G0 X10 Y30 Z30")

        self.docobj.Path = Path.Path([c, c1])
        postables = [self.docobj]

        args = "--no-header --axis-modal --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[6]
        expected = "G0 Y30.000"
        self.assertEqual(result, expected)

    def test080(self):
        """
        Test tool change
        """
        c = Path.Command("M6 T2")
        c2 = Path.Command("M3 S3000")
        self.docobj.Path = Path.Path([c, c2])
        postables = [self.docobj]

        args = "--no-header --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        self.assertEqual(gcode.splitlines()[5], "M5")
        self.assertEqual(gcode.splitlines()[6], "M6 T2 ")
        self.assertEqual(gcode.splitlines()[7], "G43 H2")
        self.assertEqual(gcode.splitlines()[8], "M3 S3000")

        # suppress TLO
        args = "--no-header --no-tlo --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        self.assertEqual(gcode.splitlines()[7], "M3 S3000")

    def test090(self):
        """
        Test comment
        """

        c = Path.Command("(comment)")

        self.docobj.Path = Path.Path([c])
        postables = [self.docobj]

        args = "--no-header --no-show-editor"
        gcode = postprocessor.export(postables, "-", args)
        result = gcode.splitlines()[5]
        expected = "(comment)"
        self.assertEqual(result, expected)
